{ Welcome to the wonderful world of XLib Unit.
  
  This unit interfaces Xlib.h and X.h. 

  Copyright (C) 2002 by Eike Lange

  Author: Eike Lange <eike@g-n-u.de>

  Documentation:
  ==============
  Of you need some documentation about XLib-Programming,
  please visit the following sites:
  (I am not responsible for the contents of these sites)
  *  http://www.tronche.com/gui/x/xlib/introduction/
  *  http://www.rahul.net/kenton/xsites.framed.html

  Different to C-Interface:
  =========================
  * Always:
    - underscores removed
    - "class" ==> "klass"
    - Element "Type"  ==> "EventType" in EventRecords
  * Record TXGcValues (XGcValues):
    Element "FunctionE" is a different name
  * TXSelectionEvent (XSelectionEvent):
    TTarget, TProperty are different names
  * TXColormapEvent (XColormapEvent): ANew is a different name
  * TXIMText (XIMText) and TXIMStringConversionText
    (XIMStringConversionText): AString is a different name
  * TXIMStatusDrawCallbackStruct (XIMStatusDrawCallbackStruct)
      "DataType" is a different name
  * Due to a name clash with constant "XLookupKeySym"
      function XLookupKeysymP is different name
  * Macros are rewritten as functions. See xlibc.c for details.
  
  To Do:
  ======
  * OOP XLib Unit (E-Mail from Markus G.)
  * Fix all those "@@" comments
  * return values of 
    TBool should be Boolean
    PChar should be String
    ...
  (If you need all this, please let me know)

  This unit is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation, version 2.

  This unit is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; see the file COPYING.LIB. If not,
  write to the Free Software Foundation, Inc., 59 Temple Place -
  Suite 330, Boston, MA 02111-1307, USA. }

{ Please note the original copyright notice on top of
  Xlib.h and X.h }

{$gnu-pascal,I-}

unit Xlib;

interface

{@section Types from X.h }
type
  TXID = MedCard;  { unsigned long in C }
  TMask = TXID;
  PAtom = ^TAtom;
  TAtom = TXID;
  TVisualID = TXID;
  TTime = TXID;
  TWindow = TXID;
  TDrawable = TXID;
  TFont = TXID;
  TPixmap = TXID;
  TCursor = TXID;
  TColormap = TXID;
  TGContext = TXID;
  TKeySym = TXID;
  TKeyCode = Byte;

{@section Consts from X.h}
{ Please do not assign anything to these identifies. They should
be treated as constants }
var
  X_PROTOCOL:           Integer; asmname '_p_X_PROTOCOL';         external;
  X_PROTOCOL_REVISION:  Integer; asmname '_p_X_PROTOCOL_REVISION';external;
  None:                 Integer; asmname '_p_None';               external;
  ParentRelative:       Integer; asmname '_p_ParentRelative';     external;
  CopyFromParent:       Integer; asmname '_p_CopyFromParent';     external;
  PointerWindow:        Integer; asmname '_p_PointerWindow';      external;
  InputFocus:           Integer; asmname '_p_InputFocus';         external;
  PointerRoot:          Integer; asmname '_p_PointerRoot';        external;
  AnyPropertyType:      Integer; asmname '_p_AnyPropertyType';    external;
  AnyKey:               Integer; asmname '_p_AnyKey';             external;
  AnyButton:            Integer; asmname '_p_AnyButton';          external;
  AllTemporary:         Integer; asmname '_p_AllTemporary';       external;
  CurrentTime:          Integer; asmname '_p_CurrentTime';        external;
  NoSymbol:             Integer; asmname '_p_NoSymbol';           external;
  
  {@subsection EVENT DEFINITIONS }
  { Input Event Masks }
  NoEventMask:          Integer; asmname '_p_NoEventMask';        external;
  KeyPressMask:         Integer; asmname '_p_KeyPressMask';       external;
  KeyReleaseMask:       Integer; asmname '_p_KeyReleaseMask';     external;
  ButtonPressMask:      Integer; asmname '_p_ButtonPressMask';    external;
  ButtonReleaseMask:    Integer; asmname '_p_ButtonReleaseMask';  external;
  EnterWindowMask:      Integer; asmname '_p_EnterWindowMask';    external;
  LeaveWindowMask:      Integer; asmname '_p_LeaveWindowMask';    external;
  PointerMotionMask:    Integer; asmname '_p_PointerMotionMask';  external;
  PointerMotionHintMask:Integer; asmname '_p_PointerMotionHintMask';  external;
  Button1MotionMask:    Integer; asmname '_p_Button1MotionMask';  external;
  Button2MotionMask:    Integer; asmname '_p_Button2MotionMask';  external;
  Button3MotionMask:    Integer; asmname '_p_Button3MotionMask';  external;
  Button4MotionMask:    Integer; asmname '_p_Button4MotionMask';  external;
  Button5MotionMask:    Integer; asmname '_p_Button5MotionMask';  external;
  ButtonMotionMask:     Integer; asmname '_p_ButtonMotionMask';   external;
  KeymapStateMask:      Integer; asmname '_p_KeymapStateMask';    external;
  ExposureMask:         Integer; asmname '_p_ExposureMask';       external;
  VisibilityChangeMask: Integer; asmname '_p_VisibilityChangeMask';   external;
  StructureNotifyMask:  Integer; asmname '_p_StructureNotifyMask';external;
  ResizeRedirectMask:   Integer; asmname '_p_ResizeRedirectMask'; external;
  SubstructureNotifyMask:   Integer; asmname '_p_SubstructureNotifyMask';  external;
  SubstructureRedirectMask: Integer; asmname '_p_SubstructureRedirectMask';external;
  FocusChangeMask:      Integer; asmname '_p_FocusChangeMask';    external;
  PropertyChangeMask:   Integer; asmname '_p_PropertyChangeMask'; external;
  ColormapChangeMask:   Integer; asmname '_p_ColormapChangeMask'; external;
  OwnerGrabButtonMask:  Integer; asmname '_p_OwnerGrabButtonMask';external;
  
  { Event names }
  KeyPress:             Integer; asmname '_p_KeyPress';           external;
  KeyRelease:           Integer; asmname '_p_KeyRelease';         external;
  ButtonPress:          Integer; asmname '_p_ButtonPress';        external;
  ButtonRelease:        Integer; asmname '_p_ButtonRelease';      external;
  MotionNotify:         Integer; asmname '_p_MotionNotify';       external;
  EnterNotify:          Integer; asmname '_p_EnterNotify';        external;
  LeaveNotify:          Integer; asmname '_p_LeaveNotify';        external;
  FocusIn:              Integer; asmname '_p_FocusIn';            external;
  FocusOut:             Integer; asmname '_p_FocusOut';           external;
  KeymapNotify:         Integer; asmname '_p_KeymapNotify';       external;
  Expose:               Integer; asmname '_p_Expose';             external;
  GraphicsExpose:       Integer; asmname '_p_GraphicsExpose';     external;
  NoExpose:             Integer; asmname '_p_NoExpose';           external;
  VisibilityNotify:     Integer; asmname '_p_VisibilityNotify';   external;
  CreateNotify:         Integer; asmname '_p_CreateNotify';       external;
  DestroyNotify:        Integer; asmname '_p_DestroyNotify';      external;
  UnmapNotify:          Integer; asmname '_p_UnmapNotify';        external;
  MapNotify:            Integer; asmname '_p_MapNotify';          external;
  MapRequest:           Integer; asmname '_p_MapRequest';         external;
  ReparentNotify:       Integer; asmname '_p_ReparentNotify';     external;
  ConfigureNotify:      Integer; asmname '_p_ConfigureNotify';    external;
  ConfigureRequest:     Integer; asmname '_p_ConfigureRequest';   external;
  GravityNotify:        Integer; asmname '_p_GravityNotify';      external;
  ResizeRequest:        Integer; asmname '_p_ResizeRequest';      external;
  CirculateNotify:      Integer; asmname '_p_CirculateNotify';    external;
  CirculateRequest:     Integer; asmname '_p_CirculateRequest';   external;
  PropertyNotify:       Integer; asmname '_p_PropertyNotify';     external;
  SelectionClear:       Integer; asmname '_p_SelectionClear';     external;
  SelectionRequest:     Integer; asmname '_p_SelectionRequest';   external;
  SelectionNotify:      Integer; asmname '_p_SelectionNotify';    external;
  ColormapNotify:       Integer; asmname '_p_ColormapNotify';     external;
  ClientMessage:        Integer; asmname '_p_ClientMessage';      external;
  MappingNotify:        Integer; asmname '_p_MappingNotify';      external;
  LASTEvent:            Integer; asmname '_p_LASTEvent';          external;
  
  { Key masks }
  ShiftMask:            Integer; asmname '_p_ShiftMask';          external;
  LockMask:             Integer; asmname '_p_LockMask';           external;
  ControlMask:          Integer; asmname '_p_ControlMask';        external;
  Mod1Mask:             Integer; asmname '_p_Mod1Mask';           external;
  Mod2Mask:             Integer; asmname '_p_Mod2Mask';           external;
  Mod3Mask:             Integer; asmname '_p_Mod3Mask';           external;
  Mod4Mask:             Integer; asmname '_p_Mod4Mask';           external;
  Mod5Mask:             Integer; asmname '_p_Mod5Mask';           external;
  
  { modifier names }
  ShiftMapIndex:        Integer; asmname '_p_ShiftMapIndex';      external;
  LockMapIndex:         Integer; asmname '_p_LockMapIndex';       external;
  ControlMapIndex:      Integer; asmname '_p_ControlMapIndex';    external;
  Mod1MapIndex:         Integer; asmname '_p_Mod1MapIndex';       external;
  Mod2MapIndex:         Integer; asmname '_p_Mod2MapIndex';       external;
  Mod3MapIndex:         Integer; asmname '_p_Mod3MapIndex';       external;
  Mod4MapIndex:         Integer; asmname '_p_Mod4MapIndex';       external;
  Mod5MapIndex:         Integer; asmname '_p_Mod5MapIndex';       external;
  
  { button masks }
  Button1Mask:          Integer; asmname '_p_Button1Mask';        external;
  Button2Mask:          Integer; asmname '_p_Button2Mask';        external;
  Button3Mask:          Integer; asmname '_p_Button3Mask';        external;
  Button4Mask:          Integer; asmname '_p_Button4Mask';        external;
  Button5Mask:          Integer; asmname '_p_Button5Mask';        external;
  AnyModifier:          Integer; asmname '_p_AnyModifier';        external;
  
  { button names }
  Button1:              Integer; asmname '_p_Button1';            external;
  Button2:              Integer; asmname '_p_Button2';            external;
  Button3:              Integer; asmname '_p_Button3';            external;
  Button4:              Integer; asmname '_p_Button4';            external;
  Button5:              Integer; asmname '_p_Button5';            external;
  
  { Notify modes }
  NotifyNormal:         Integer; asmname '_p_NotifyNormal';       external;
  NotifyGrab:           Integer; asmname '_p_NotifyGrab';         external;
  NotifyUngrab:         Integer; asmname '_p_NotifyUngrab';       external;
  NotifyWhileGrabbed:   Integer; asmname '_p_NotifyWhileGrabbed'; external;
  NotifyHint:           Integer; asmname '_p_NotifyHint';         external;
  
  { Notify detail }
  NotifyAncestor:       Integer; asmname '_p_NotifyAncestor';     external;
  NotifyVirtual:        Integer; asmname '_p_NotifyVirtual';      external;
  NotifyInferior:       Integer; asmname '_p_NotifyInferior';     external;
  NotifyNonlinear:      Integer; asmname '_p_NotifyNonlinear';    external;
  NotifyNonlinearVirtual:   Integer; asmname '_p_NotifyNonlinearVirtual'; external;
  NotifyPointer:        Integer; asmname '_p_NotifyPointer';      external;
  NotifyPointerRoot:    Integer; asmname '_p_NotifyPointerRoot';  external;
  NotifyDetailNone:     Integer; asmname '_p_NotifyDetailNone';   external;
  
  { Visibility notify }
  VisibilityUnobscured: Integer; asmname '_p_VisibilityUnobscured'; external;
  VisibilityPartiallyObscured:  Integer; asmname '_p_VisibilityPartiallyObscured';  external;
  VisibilityFullyObscured:      Integer; asmname '_p_VisibilityFullyObscured';  external;
  
  { Circulation reques }
  PlaceOnTop:           Integer; asmname '_p_PlaceOnTop';         external;
  PlaceOnBottom:        Integer; asmname '_p_PlaceOnBottom';      external;
  
  { protocol families }
  FamilyInternet:       Integer; asmname '_p_FamilyInternet';     external;
  FamilyDECnet:         Integer; asmname '_p_FamilyDECnet';       external;
  FamilyChaos:          Integer; asmname '_p_FamilyChaos';        external;
  
  { Property notification }
  PropertyNewValue:     Integer; asmname '_p_PropertyNewValue';   external;
  PropertyDelete:       Integer; asmname '_p_PropertyDelete';     external;
  
  { Color Map notification }
  ColormapUninstalled:  Integer; asmname '_p_ColormapUninstalled';external;
  ColormapInstalled:    Integer; asmname '_p_ColormapInstalled';  external;
  
  { GrabPointer, GrabButton, GrabKeyboard, GrabKey Modes }
  GrabModeSync:         Integer; asmname '_p_GrabModeSync';       external;
  GrabModeAsync:        Integer; asmname '_p_GrabModeAsync';      external;
  
  { GrabPointer, GrabKeyboard reply status }
  GrabSuccess:          Integer; asmname '_p_GrabSuccess';        external;
  AlreadyGrabbed:       Integer; asmname '_p_AlreadyGrabbed';     external;
  GrabInvalidTime:      Integer; asmname '_p_GrabInvalidTime';    external;
  GrabNotViewable:      Integer; asmname '_p_GrabNotViewable';    external;
  GrabFrozen:           Integer; asmname '_p_GrabFrozen';         external;
  
  { AllowEvents modes }
  AsyncPointer:         Integer; asmname '_p_AsyncPointer';       external;
  SyncPointer:          Integer; asmname '_p_SyncPointer';        external;
  ReplayPointer:        Integer; asmname '_p_ReplayPointer';      external;
  AsyncKeyboard:        Integer; asmname '_p_AsyncKeyboard';      external;
  SyncKeyboard:         Integer; asmname '_p_SyncKeyboard';       external;
  ReplayKeyboard:       Integer; asmname '_p_ReplayKeyboard';     external;
  AsyncBoth:            Integer; asmname '_p_AsyncBoth';          external;
  SyncBoth:             Integer; asmname '_p_SyncBoth';           external;
  
  { Used in SetInputFocus, GetInputFocus }
  RevertToNone:         Integer; asmname '_p_RevertToNone';       external;
  RevertToPointerRoot:  Integer; asmname '_p_RevertToPointerRoot';external;
  RevertToParent:       Integer; asmname '_p_RevertToParent';     external;
  
  {@subsection ERROR CODES }
  
  Success:              Integer; asmname '_p_Success';            external;
  BadRequest:           Integer; asmname '_p_BadRequest';         external;
  BadValue:             Integer; asmname '_p_BadValue';           external;
  BadWindow:            Integer; asmname '_p_BadWindow';          external;
  BadPixmap:            Integer; asmname '_p_BadPixmap';          external;
  BadAtom:              Integer; asmname '_p_BadAtom';            external;
  BadCursor:            Integer; asmname '_p_BadCursor';          external;
  BadFont:              Integer; asmname '_p_BadFont';            external;
  BadMatch:             Integer; asmname '_p_BadMatch';           external;
  BadDrawable:          Integer; asmname '_p_BadDrawable';        external;
  BadAccess:            Integer; asmname '_p_BadAccess';          external;
  BadAlloc:             Integer; asmname '_p_BadAlloc';           external;
  BadColor:             Integer; asmname '_p_BadColor';           external;
  BadGC:                Integer; asmname '_p_BadGC';              external;
  BadIDChoice:          Integer; asmname '_p_BadIDChoice';        external;
  BadName:              Integer; asmname '_p_BadName';            external;
  BadLength:            Integer; asmname '_p_BadLength';          external;
  BadImplementation:    Integer; asmname '_p_BadImplementation';  external;
  FirstExtensionError:  Integer; asmname '_p_FirstExtensionError';external;
  LastExtensionError:   Integer; asmname '_p_LastExtensionError'; external;
  
  {@subsection WINDOW DEFINITIONS }
  { Window classes used by CreateWindow }
  InputOutput:          Integer; asmname '_p_InputOutput';        external;
  InputOnly:            Integer; asmname '_p_InputOnly';          external;
  
  { Window attributes for CreateWindow and ChangeWindowAttributes }
  CWBackPixmap:         Integer; asmname '_p_CWBackPixmap';       external;
  CWBackPixel:          Integer; asmname '_p_CWBackPixel';        external;
  CWBorderPixmap:       Integer; asmname '_p_CWBorderPixmap';     external;
  CWBorderPixel:        Integer; asmname '_p_CWBorderPixel';      external;
  CWBitGravity:         Integer; asmname '_p_CWBitGravity';       external;
  CWWinGravity:         Integer; asmname '_p_CWWinGravity';       external;
  CWBackingStore:       Integer; asmname '_p_CWBackingStore';     external;
  CWBackingPlanes:      Integer; asmname '_p_CWBackingPlanes';    external;
  CWBackingPixel:       Integer; asmname '_p_CWBackingPixel';     external;
  CWOverrideRedirect:   Integer; asmname '_p_CWOverrideRedirect'; external;
  CWSaveUnder:          Integer; asmname '_p_CWSaveUnder';        external;
  CWEventMask:          Integer; asmname '_p_CWEventMask';        external;
  CWDontPropagate:      Integer; asmname '_p_CWDontPropagate';    external;
  CWColormap:           Integer; asmname '_p_CWColormap';         external;
  CWCursor:             Integer; asmname '_p_CWCursor';           external;
  
  { ConfigureWindow structure }
  CWX:                  Integer; asmname '_p_CWX';                external;
  CWY:                  Integer; asmname '_p_CWY';                external;
  CWWidth:              Integer; asmname '_p_CWWidth';            external;
  CWHeight:             Integer; asmname '_p_CWHeight';           external;
  CWBorderWidth:        Integer; asmname '_p_CWBorderWidth';      external;
  CWSibling:            Integer; asmname '_p_CWSibling';          external;
  CWStackMode:          Integer; asmname '_p_CWStackMode';        external;
  
  { Bit Gravity }
  ForgetGravity:        Integer; asmname '_p_ForgetGravity';      external;
  NorthWestGravity:     Integer; asmname '_p_NorthWestGravity';   external;
  NorthGravity:         Integer; asmname '_p_NorthGravity';       external;
  NorthEastGravity:     Integer; asmname '_p_NorthEastGravity';   external;
  WestGravity:          Integer; asmname '_p_WestGravity';        external;
  CenterGravity:        Integer; asmname '_p_CenterGravity';      external;
  EastGravity:          Integer; asmname '_p_EastGravity';        external;
  SouthWestGravity:     Integer; asmname '_p_SouthWestGravity';   external;
  SouthGravity:         Integer; asmname '_p_SouthGravity';       external;
  SouthEastGravity:     Integer; asmname '_p_SouthEastGravity';   external;
  StaticGravity:        Integer; asmname '_p_StaticGravity';      external;

  { Window gravity + bit gravity above }
  UnmapGravity:         Integer; asmname '_p_UnmapGravity';       external;

  { Used in CreateWindow for backing-store hint }
  NotUseful:            Integer; asmname '_p_NotUseful';          external;
  WhenMapped:           Integer; asmname '_p_WhenMapped';         external;
  Always:               Integer; asmname '_p_Always';             external;

  { Used in GetWindowAttributes reply }
  IsUnmapped:           Integer; asmname '_p_IsUnmapped';         external;
  IsUnviewable:         Integer; asmname '_p_IsUnviewable';       external;
  IsViewable:           Integer; asmname '_p_IsViewable';         external;
  
  { Used in ChangeSaveSet }
  SetModeInsert:        Integer; asmname '_p_SetModeInsert';      external;
  SetModeDelete:        Integer; asmname '_p_SetModeDelete';      external;
  
  { Used in ChangeCloseDownMode }
  DestroyAll:           Integer; asmname '_p_DestroyAll';         external;
  RetainPermanent:      Integer; asmname '_p_RetainPermanent';    external;
  RetainTemporary:      Integer; asmname '_p_RetainTemporary';    external;
  
  { Window stacking method (in configureWindow)  }
  Above:                Integer; asmname '_p_Above';              external;
  Below:                Integer; asmname '_p_Below';              external;
  TopIf:                Integer; asmname '_p_TopIf';              external;
  BottomIf:             Integer; asmname '_p_BottomIf';           external;
  Opposite:             Integer; asmname '_p_Opposite';           external;
  
  { Circulation direction }
  RaiseLowest:          Integer; asmname '_p_RaiseLowest';        external;
  LowerHighest:         Integer; asmname '_p_LowerHighest';       external;
  
  { Property modes }
  PropModeReplace:      Integer; asmname '_p_PropModeReplace';    external;
  PropModePrepend:      Integer; asmname '_p_PropModePrepend';    external;
  PropModeAppend:       Integer; asmname '_p_PropModeAppend';     external;
  
  {@subsection GRAPHICS DEFINITIONS }
  { graphics functions, as in GC.alu }
  GXclear:              Integer; asmname '_p_GXclear';            external;
  GXand:                Integer; asmname '_p_GXand';              external;
  GXandReverse:         Integer; asmname '_p_GXandReverse';       external;
  GXcopy:               Integer; asmname '_p_GXcopy';             external;
  GXandInverted:        Integer; asmname '_p_GXandInverted';      external;
  GXnoop:               Integer; asmname '_p_GXnoop';             external;
  GXxor:                Integer; asmname '_p_GXxor';              external;
  GXor:                 Integer; asmname '_p_GXor';               external;
  GXnor:                Integer; asmname '_p_GXnor';              external;
  GXequiv:              Integer; asmname '_p_GXequiv';            external;
  GXinvert:             Integer; asmname '_p_GXinvert';           external;
  GXorReverse:          Integer; asmname '_p_GXorReverse';        external;
  GXcopyInverted:       Integer; asmname '_p_GXcopyInverted';     external;
  GXorInverted:         Integer; asmname '_p_GXorInverted';       external;
  GXnand:               Integer; asmname '_p_GXnand';             external;
  GXset:                Integer; asmname '_p_GXset';              external;
  
  { LineStyle }
  LineSolid:            Integer; asmname '_p_LineSolid';          external;
  LineOnOffDash:        Integer; asmname '_p_LineOnOffDash';      external;
  LineDoubleDash:       Integer; asmname '_p_LineDoubleDash';     external;
  
  { capStyle }
  CapNotLast:           Integer; asmname '_p_CapNotLast';         external;
  CapButt:              Integer; asmname '_p_CapButt';            external;
  CapRound:             Integer; asmname '_p_CapRound';           external;
  CapProjecting:        Integer; asmname '_p_CapProjecting';      external;

  { joinStyle }
  JoinMiter:            Integer; asmname '_p_JoinMiter';          external;
  JoinRound:            Integer; asmname '_p_JoinRound';          external;
  JoinBevel:            Integer; asmname '_p_JoinBevel';          external;
  
  { fillStyle }
  FillSolid:            Integer; asmname '_p_FillSolid';          external;
  FillTiled:            Integer; asmname '_p_FillTiled';          external;
  FillStippled:         Integer; asmname '_p_FillStippled';       external;
  FillOpaqueStippled:   Integer; asmname '_p_FillOpaqueStippled'; external;
  
  { fillRule }
  EvenOddRule:          Integer; asmname '_p_EvenOddRule';        external;
  WindingRule:          Integer; asmname '_p_WindingRule';        external;

  { subwindow mode }
  ClipByChildren:       Integer; asmname '_p_ClipByChildren';     external;
  IncludeInferiors:     Integer; asmname '_p_IncludeInferiors';   external;

  { SetClipRectangles ordering }
  Unsorted:             Integer; asmname '_p_Unsorted';           external;
  YSorted:              Integer; asmname '_p_YSorted';            external;
  YXSorted:             Integer; asmname '_p_YXSorted';           external;
  YXBanded:             Integer; asmname '_p_YXBanded';           external;

  { CoordinateMode for drawing routines }
  CoordModeOrigin:      Integer; asmname '_p_CoordModeOrigin';    external;
  CoordModePrevious:    Integer; asmname '_p_CoordModePrevious';  external;

  { Polygon shapes }
  Complex:              Integer; asmname '_p_Complex';            external;
  Nonconvex:            Integer; asmname '_p_Nonconvex';          external;
  Convex:               Integer; asmname '_p_Convex';             external;

  { Arc modes for PolyFillArc }
  ArcChord:             Integer; asmname '_p_ArcChord';           external;
  ArcPieSlice:          Integer; asmname '_p_ArcPieSlice';        external;

  { GC components: masks used in CreateGC, CopyGC, ChangeGC, OR'ed into
    GC.stateChanges }
  GCFunction:           Integer; asmname '_p_GCFunction';         external;
  GCPlaneMask:          Integer; asmname '_p_GCPlaneMask';        external;
  GCForeground:         Integer; asmname '_p_GCForeground';       external;
  GCBackground:         Integer; asmname '_p_GCBackground';       external;
  GCLineWidth:          Integer; asmname '_p_GCLineWidth';        external;
  GCLineStyle:          Integer; asmname '_p_GCLineStyle';        external;
  GCCapStyle:           Integer; asmname '_p_GCCapStyle';         external;
  GCJoinStyle:          Integer; asmname '_p_GCJoinStyle';        external;
  GCFillStyle:          Integer; asmname '_p_GCFillStyle';        external;
  GCFillRule:           Integer; asmname '_p_GCFillRule';         external;
  GCTile:               Integer; asmname '_p_GCTile';             external;
  GCStipple:            Integer; asmname '_p_GCStipple';          external;
  GCTileStipXOrigin:    Integer; asmname '_p_GCTileStipXOrigin';  external;
  GCTileStipYOrigin:    Integer; asmname '_p_GCTileStipYOrigin';  external;
  GCFont:               Integer; asmname '_p_GCFont';             external;
  GCSubwindowMode:      Integer; asmname '_p_GCSubwindowMode';    external;
  GCGraphicsExposures:  Integer; asmname '_p_GCGraphicsExposures';external;
  GCClipXOrigin:        Integer; asmname '_p_GCClipXOrigin';      external;
  GCClipYOrigin:        Integer; asmname '_p_GCClipYOrigin';      external;
  GCClipMask:           Integer; asmname '_p_GCClipMask';         external;
  GCDashOffset:         Integer; asmname '_p_GCDashOffset';       external;
  GCDashList:           Integer; asmname '_p_GCDashList';         external;
  GCArcMode:            Integer; asmname '_p_GCArcMode';          external;
  GCLastBit:            Integer; asmname '_p_GCLastBit';          external;

  {@subsection FONTS }
  { used in QueryFont -- draw direction }
  FontLeftToRight:      Integer; asmname '_p_FontLeftToRight';    external;
  FontRightToLeft:      Integer; asmname '_p_FontRightToLeft';    external;
  FontChange:           Integer; asmname '_p_FontChange';         external;
  
  {@subsection IMAGING }
  { ImageFormat -- PutImage, GetImage }
  XYBitmap:             Integer; asmname '_p_XYBitmap';           external;
  XYPixmap:             Integer; asmname '_p_XYPixmap';           external;
  ZPixmap:              Integer; asmname '_p_ZPixmap';            external;
  
  {@subsection COLOR MAP STUFF }
  { For CreateColormap }
  AllocNone:            Integer; asmname '_p_AllocNone';          external;
  AllocAll:             Integer; asmname '_p_AllocAll';           external;

  { Flags used in StoreNamedColor, StoreColors }
  DoRed:                Integer; asmname '_p_DoRed';              external;
  DoGreen:              Integer; asmname '_p_DoGreen';            external;
  DoBlue:               Integer; asmname '_p_DoBlue';             external;

  {@subsection CURSOR STUFF }
  { QueryBestSize Class }
  CursorShape:          Integer; asmname '_p_CursorShape';        external;
  TileShape:            Integer; asmname '_p_TileShape';          external;
  StippleShape:         Integer; asmname '_p_StippleShape';       external;

  {@subsection KEYBOARD/POINTER STUFF }
  AutoRepeatModeOff:    Integer; asmname '_p_AutoRepeatModeOff';  external;
  AutoRepeatModeOn:     Integer; asmname '_p_AutoRepeatModeOn';   external;
  AutoRepeatModeDefault:Integer; asmname '_p_AutoRepeatModeDefault';  external;
  LedModeOff:           Integer; asmname '_p_LedModeOff';         external;
  LedModeOn:            Integer; asmname '_p_LedModeOn';          external;

  { masks for ChangeKeyboardControl }
  KBKeyClickPercent:    Integer; asmname '_p_KBKeyClickPercent';  external;
  KBBellPercent:        Integer; asmname '_p_KBBellPercent';      external;
  KBBellPitch:          Integer; asmname '_p_KBBellPitch';        external;
  KBBellDuration:       Integer; asmname '_p_KBBellDuration';     external;
  KBLed:                Integer; asmname '_p_KBLed';              external;
  KBLedMode:            Integer; asmname '_p_KBLedMode';          external;
  KBKey:                Integer; asmname '_p_KBKey';              external;
  KBAutoRepeatMode:     Integer; asmname '_p_KBAutoRepeatMode';   external;
  MappingSuccess:       Integer; asmname '_p_MappingSuccess';     external;
  MappingBusy:          Integer; asmname '_p_MappingBusy';        external;
  MappingFailed:        Integer; asmname '_p_MappingFailed';      external;
  MappingModifier:      Integer; asmname '_p_MappingModifier';    external;
  MappingKeyboard:      Integer; asmname '_p_MappingKeyboard';    external;
  MappingPointer:       Integer; asmname '_p_MappingPointer';     external;

  {@subsection SCREEN SAVER STUFF }
  DontPreferBlanking:   Integer; asmname '_p_DontPreferBlanking'; external;
  PreferBlanking:       Integer; asmname '_p_PreferBlanking';     external;
  DefaultBlanking:      Integer; asmname '_p_DefaultBlanking';    external;
  DisableScreenSaver:   Integer; asmname '_p_DisableScreenSaver'; external;
  DisableScreenInterval:    Integer; asmname '_p_DisableScreenInterval';  external;
  DontAllowExposures:   Integer; asmname '_p_DontAllowExposures'; external;
  AllowExposures:       Integer; asmname '_p_AllowExposures';     external;
  DefaultExposures:     Integer; asmname '_p_DefaultExposures';   external;

  { for ForceScreenSaver }
  ScreenSaverReset:     Integer; asmname '_p_ScreenSaverReset';   external;
  ScreenSaverActive:    Integer; asmname '_p_ScreenSaverActive';  external;

  {@subsection HOSTS AND CONNECTIONS }
  { for ChangeHosts }
  HostInsert:           Integer; asmname '_p_HostInsert';         external;
  HostDelete:           Integer; asmname '_p_HostDelete';         external;

  { for ChangeAccessControl }
  EnableAccess:         Integer; asmname '_p_EnableAccess';       external;
  DisableAccess:        Integer; asmname '_p_DisableAccess';      external;

  { Display classes  used in opening the connection }
  StaticGray:           Integer; asmname '_p_StaticGray';         external;
  GrayScale:            Integer; asmname '_p_GrayScale';          external;
  StaticColor:          Integer; asmname '_p_StaticColor';        external;
  PseudoColor:          Integer; asmname '_p_PseudoColor';        external;
  TrueColor:            Integer; asmname '_p_TrueColor';          external;
  DirectColor:          Integer; asmname '_p_DirectColor';        external;

  { Byte order  used in imageByteOrder and bitmapBitOrder }
  LSBFirst:             Integer; asmname '_p_LSBFirst';           external;
  MSBFirst:             Integer; asmname '_p_MSBFirst';           external;

{@section Consts from Xlib.h}
{ Please do not assign anything to these identifies. They should
be treated as constants }
var
  XTrue:                Integer; asmname '_p_True';               external;
  XFalse:               Integer; asmname '_p_False';              external;
  QueuedAlready:        Integer; asmname '_p_QueuedAlready';      external;
  QueuedAfterReading:   Integer; asmname '_p_QueuedAfterReading'; external;
  QueuedAfterFlush:     Integer; asmname '_p_QueuedAfterFlush';   external;
  XIMPreeditArea:       Integer; asmname '_p_XIMPreeditArea';     external;
  XIMPreeditCallbacks:  Integer; asmname '_p_XIMPreeditCallbacks';external;
  XIMPreeditPosition:   Integer; asmname '_p_XIMPreeditPosition'; external;
  XIMPreeditNothing:    Integer; asmname '_p_XIMPreeditNothing';  external;
  XIMPreeditNone:       Integer; asmname '_p_XIMPreeditNone';     external;
  XIMStatusArea:        Integer; asmname '_p_XIMStatusArea';      external;
  XIMStatusCallbacks:   Integer; asmname '_p_XIMStatusCallbacks'; external;
  XIMStatusNothing:     Integer; asmname '_p_XIMStatusNothing';   external;
  XIMStatusNone:        Integer; asmname '_p_XIMStatusNone';      external;
  XBufferOverflow:      Integer; asmname '_p_XBufferOverflow';    external;
  XLookupNone:          Integer; asmname '_p_XLookupNone';        external;
  XLookupChars:         Integer; asmname '_p_XLookupChars';       external;
  XLookupKeySym:        Integer; asmname '_p_XLookupKeySym';      external;
  XLookupBoth:          Integer; asmname '_p_XLookupBoth';        external;
  XIMReverse:           Integer; asmname '_p_XIMReverse';         external;
  XIMUnderline:         Integer; asmname '_p_XIMUnderline';       external;
  XIMHighlight:         Integer; asmname '_p_XIMHighlight';       external;
  XIMPrimary:           Integer; asmname '_p_XIMPrimary';         external;
  XIMSecondary:         Integer; asmname '_p_XIMSecondary';       external;
  XIMTertiary:          Integer; asmname '_p_XIMTertiary';        external;
  XIMVisibleToForward:  Integer; asmname '_p_XIMVisibleToForward';  external;
  XIMVisibleToBackword: Integer; asmname '_p_XIMVisibleToBackword'; external;
  XIMVisibleToCenter:   Integer; asmname '_p_XIMVisibleToCenter'; external;
  XNRequiredCharSet:    PChar;   asmname '_p_XNRequiredCharSet';  external;
  XNQueryOrientation:   PChar;   asmname '_p_XNQueryOrientation'; external;
  XNBaseFontName:       PChar;   asmname '_p_XNBaseFontName';     external;
  XNOMAutomatic:        PChar;   asmname '_p_XNOMAutomatic';      external;
  XNMissingCharSet:     PChar;   asmname '_p_XNMissingCharSet';   external;
  XNDefaultString:      PChar;   asmname '_p_XNDefaultString';    external;
  XNOrientation:        PChar;   asmname '_p_XNOrientation';      external;
  XNDirectionalDependentDrawing:PChar; asmname '_p_XNDirectionalDependentDrawing'; external;
  XNContextualDrawing:  PChar; asmname '_p_XNContextualDrawing';  external;
  XNFontInfo:           PChar; asmname '_p_XNFontInfo';           external;
  XNVaNestedList:       PChar; asmname '_p_XNVaNestedList';       external;
  XNQueryInputStyle:    PChar; asmname '_p_XNQueryInputStyle';    external;
  XNClientWindow:       PChar; asmname '_p_XNClientWindow';       external;
  XNInputStyle:         PChar; asmname '_p_XNInputStyle';         external;
  XNFocusWindow:        PChar; asmname '_p_XNFocusWindow';        external;
  XNResourceName:       PChar; asmname '_p_XNResourceName';       external;
  XNResourceClass:      PChar; asmname '_p_XNResourceClass';      external;
  XNGeometryCallback:   PChar; asmname '_p_XNGeometryCallback';   external;
  XNDestroyCallback:    PChar; asmname '_p_XNDestroyCallback';    external;
  XNFilterEvents:       PChar; asmname '_p_XNFilterEvents';       external;
  XNPreeditStartCallback:       PChar; asmname '_p_XNPreeditStartCallback'; external;
  XNPreeditDoneCallback:        PChar; asmname '_p_XNPreeditDoneCallback';  external;
  XNPreeditDrawCallback:        PChar; asmname '_p_XNPreeditDrawCallback';  external;
  XNPreeditCaretCallback:       PChar; asmname '_p_XNPreeditCaretCallback'; external;
  XNPreeditStateNotifyCallback: PChar; asmname '_p_XNPreeditStateNotifyCallback'; external;
  XNPreeditAttributes:          PChar; asmname '_p_XNPreeditAttributes';    external;
  XNStatusStartCallback:        PChar; asmname '_p_XNStatusStartCallback';  external;
  XNStatusDoneCallback: PChar; asmname '_p_XNStatusDoneCallback'; external;
  XNStatusDrawCallback: PChar; asmname '_p_XNStatusDrawCallback'; external;
  XNStatusAttributes:   PChar; asmname '_p_XNStatusAttributes';   external;
  XNArea:               PChar; asmname '_p_XNArea';               external;
  XNAreaNeeded:         PChar; asmname '_p_XNAreaNeeded';         external;
  XNSpotLocation:       PChar; asmname '_p_XNSpotLocation';       external;
  XNColormap:           PChar; asmname '_p_XNColormap';           external;
  XNStdColormap:        PChar; asmname '_p_XNStdColormap';        external;
  XNForeground:         PChar; asmname '_p_XNForeground';         external;
  XNBackground:         PChar; asmname '_p_XNBackground';         external;
  XNBackgroundPixmap:   PChar; asmname '_p_XNBackgroundPixmap';   external;
  XNFontSet:            PChar; asmname '_p_XNFontSet';            external;
  XNLineSpace:          PChar; asmname '_p_XNLineSpace';          external;
  XNCursor:             PChar; asmname '_p_XNCursor';             external;
  XNQueryIMValuesList:  PChar; asmname '_p_XNQueryIMValuesList';  external;
  XNQueryICValuesList:  PChar; asmname '_p_XNQueryICValuesList';  external;
  XNVisiblePosition:    PChar; asmname '_p_XNVisiblePosition';    external;
  XNR6PreeditCallback:  PChar; asmname '_p_XNR6PreeditCallback';  external;
  XNStringConversionCallback:   PChar; asmname '_p_XNStringConversionCallback'; external;
  XNStringConversion:   PChar; asmname '_p_XNStringConversion';   external;
  XNResetState:         PChar; asmname '_p_XNResetState';         external;
  XNHotKey:             PChar; asmname '_p_XNHotKey';             external;
  XNHotKeyState:        PChar; asmname '_p_XNHotKeyState';        external;
  XNPreeditState:       PChar; asmname '_p_XNPreeditState';       external;
  XNSeparatorofNestedList:      PChar; asmname '_p_XNSeparatorofNestedList';    external;
  XIMPreeditUnKnown:    Integer; asmname '_p_XIMPreeditUnKnown';  external;
  XIMPreeditEnable:     Integer; asmname '_p_XIMPreeditEnable';   external;
  XIMPreeditDisable:    Integer; asmname '_p_XIMPreeditDisable';  external;
  XIMInitialState:      Integer; asmname '_p_XIMInitialState';    external;
  XIMPreserveState:     Integer; asmname '_p_XIMPreserveState';   external;
  XIMStringConversionLeftEdge:  Integer; asmname '_p_XIMStringConversionLeftEdge';  external;
  XIMStringConversionRightEdge: Integer; asmname '_p_XIMStringConversionRightEdge'; external;
  XIMStringConversionTopEdge:   Integer; asmname '_p_XIMStringConversionTopEdge';   external;
  XIMStringConversionBottomEdge:Integer; asmname '_p_XIMStringConversionBottomEdge';external;
  XIMStringConversionConcealed: Integer; asmname '_p_XIMStringConversionConcealed'; external;
  XIMStringConversionWrapped:   Integer; asmname '_p_XIMStringConversionWrapped';   external;
  XIMStringConversionBuffer:    Integer; asmname '_p_XIMStringConversionBuffer';external;
  XIMStringConversionLine:      Integer; asmname '_p_XIMStringConversionLine';  external;
  XIMStringConversionWord:      Integer; asmname '_p_XIMStringConversionWord';  external;
  XIMStringConversionChar:      Integer; asmname '_p_XIMStringConversionChar';  external;
  XIMStringConversionSubstitution:  Integer; asmname '_p_XIMStringConversionSubstitution'; external;
  XIMStringConversionRetrieval: Integer; asmname '_p_XIMStringConversionRetrieval'; external;
  XIMHotKeyStateON:             Integer; asmname '_p_XIMHotKeyStateON';   external;
  XIMHotKeyStateOFF:            Integer; asmname '_p_XIMHotKeyStateOFF';  external;

  { consts from XOrientation }
  XOMOrientation_LTR_TTB: Integer;  asmname '_p_XOMOrientation_LTR_TTB';  external;
  XOMOrientation_RTL_TTB: Integer;  asmname '_p_XOMOrientation_RTL_TTB';  external;
  XOMOrientation_TTB_LTR: Integer;  asmname '_p_XOMOrientation_TTB_LTR';  external;
  XOMOrientation_TTB_RTL: Integer;  asmname '_p_XOMOrientation_TTB_RTL';  external;
  XOMOrientation_Context: Integer;  asmname '_p_XOMOrientation_Context';  external;

  { consts from XIMCaretDirection }
  XIMForwardChar:         Integer;  asmname '_p_XIMForwardChar';  external;
  XIMBackwardChar:        Integer;  asmname '_p_XIMBackwardChar'; external;
  XIMForwardWord:         Integer;  asmname '_p_XIMForwardWord';  external;
  XIMBackwardWord:        Integer;  asmname '_p_XIMForwardWord';  external;
  XIMCaretUp:             Integer;  asmname '_p_XIMCaretUp';      external;
  XIMCaretDown:           Integer;  asmname '_p_XIMCaretDown';    external;
  XIMNextLine:            Integer;  asmname '_p_XIMNextLine';     external;
  XIMPreviousLine:        Integer;  asmname '_p_XIMPreviousLine'; external;
  XIMLineStart:           Integer;  asmname '_p_XIMLineStart';    external;
  XIMLineEnd:             Integer;  asmname '_p_XIMLineEnd';      external;
  XIMAbsolutePosition:    Integer;  asmname '_p_XIMAbsolutePosition';     external;
  XIMDontChange:          Integer;  asmname '_p_XIMDontChange';   external;

  { consts from XIMCaretStyle }
  XIMIsInvisible:         Integer;  asmname '_p_XIMIsInvisible';  external;
  XIMIsPrimary:           Integer;  asmname '_p_XIMIsPrimary';    external;
  XIMIsSecondary:         Integer;  asmname '_p_XIMIsSecondary';  external;

  { consts from XIMStatusDataType }
  XIMTextType:            Integer;  asmname '_p_XIMTextType';     external;
  XIMBitmapType:          Integer;  asmname '_p_XIMBitmapType';   external;

{@section Types from Xlib.h}
type
  TBool = Integer;
  TStatus = TBool;
  PXPointer = ^TXPointer;
  TXPointer = ^ByteInt;
  PWCharT = ^TWCharT;  { Wide Character }
  TWCharT = MedCard;
  TXIMStyle = MedCard;
  PXIMFeedback = ^TXIMFeedback;
  TXIMFeedback = MedCard;
  TXIMPreeditState = MedCard;
  TXIMStringConversionPosition = ShortCard;
  TXIMStringConversionType = ShortCard;
  TXIMResetState = MedCard;
  PXIMStringConversionFeedback = ^TXIMStringConversionFeedback;
  TXIMStringConversionFeedback = MedCard;
  TXIMStringConversionOperation = ShortCard;
  TXIMHotKeyState = MedCard;

  { XExtData }
  PXExtData = ^TXExtData;
  PFreePrivateProc = ^function (extension: PXExtData): Integer;
  TXExtData = record
    number: Integer;
    next: PXExtData;
    FreePrivate: PFreePrivateProc;
    PrivateData: TXPointer
  end;
  
  { XExtCodes }
  PXExtCodes = ^TXExtCodes;
  TXExtCodes = record
    Extension, MajorOpcode, FirstEvent, FirstError: Integer
  end;
  
  { XPixmapFormatValues }
  PXPixmapFormatValues = ^TXPixmapFormatValues;
  TXPixmapFormatValues = record
    Depth, BitsPerPixel, ScanlinePad: Integer
  end;  
  
  { XGCValues }
  PXGcValues = ^TXGcValues;
  TXGcValues = record
    FunctionE: Integer;  {!! Different name }
    PlaneMask, Foreground, Background: MedCard;
    LineWidth, LineStyle: Integer;
    CapStyle: Integer;
    JoinStyle, FillStyle, FillRule: Integer;
    ArcMode: Integer;
    Tile, Stripple: TPixmap;
    TsXOrigin, TsYOrigin: Integer;
    font: TFont;
    SubwindowMode: Integer;
    GraphicsExposures: TBool;
    ClipXOrigin, ClipYOrigin: Integer;
    ClipMask: TPixmap;
    DashOffset: Integer;
    Dashes: Char
  end;

  { GC* should be treated as opaque }
  PGc = Pointer;
  
  { Visual }
  PVisual = ^TVisual;
  TVisual = record
    ExtData: PXExtData;
    VisualID: TVisualID;
    klass: Integer;  {!! Different Name }
    RedMask, GreenMask, BlueMask: MedCard;
    BitsPerRGB: Integer;
    MapEntries: Integer
  end;

  { Depth }
const
  VisualsArrayMax = High (Integer) div Sizeof (TVisual);
type
  PDepth = ^TDepth;
  PVisualsArray = ^TVisualsArray;
  TVisualsArray = array [0 .. VisualsArrayMax] of TVisual;
  TDepth = record
    Depth: Integer;
    NVisuals: Integer;
    Visuals: PVisualsArray;  
  end;

  { Display should be treated as opaque.
    The underlaying record is private to X, so I did not
    interface it. }
  PDisplay = Pointer;

  { Screen }
const
  DepthsArrayMax = High (Integer) div Sizeof (TDepth);
type
  PScreen = ^TScreen;
  PDepthsArray = ^TDepthsArray;
  TDepthsArray = array [0 .. DepthsArrayMax] of TDepth;
  TScreen = record
    ExtData: PXExtData;
    Display: PDisplay;
    Root: TWindow;
    Width, Height: Integer;
    MWidth, MHeight: Integer;
    NDepths: Integer;
    Depths: PDepthsArray;  
    RootDepth: Integer;
    RootVisual: PVisual;
    DefaultGc: PGc;
    CMap: TColormap;
    WhitePixel, BlackPixel: MedCard;
    MaxMaps, MinMaps: Integer;
    BackingStore: Integer;
    SaveUnders: TBool;
    RootInputMask: MedInt
  end;

  { ScreenFormat }
  PScreenFormat = ^TScreenFormat;
  TScreenFormat = record
    ExtData: PXExtData;
    Depth, BitsPerPixel, ScanlinePad: Integer
  end;
  
  { XWindowAttributes }
  PXWindowAttributes = ^TXWindowAttributes;
  TXWindowAttributes = record
    X, Y, Width, Height, BorderWidth, Depth: Integer;
    Visual: PVisual;
    Root: TWindow;
    Klass: Integer;  {!! Different Name }
    BitGravity, WinGravity, BackingStore: Integer;
    BackingPlanes, BackingPixel: MedCard;
    SaveUnder: TBool;
    ColorMap: TColorMap;
    MapInstalled: TBool;
    MapState: Integer;
    AllEventMasks, YourEventMask, DoNotPropagateMask: MedInt;
    OverrideRedirect: TBool;
    Screen: PScreen  { Back Pointer to correct screen }
  end;
    
  { XSetWindowAttributes }
  PXSetWindowAttributes = ^TXSetWindowAttributes;
  TXSetWindowAttributes = record
    BackgroundPixmap: TPixmap;
    BackgroundPixel: MedCard;
    BorderPixmap: TPixmap;
    BorderPixel: MedCard;
    BitGravity, WinGravity: Integer;
    BackingStore: Integer;
    BackingPlanes, BackingPixel: MedCard;
    SaveUnder: TBool;
    EventMask, DoNotPropagateMask: MedInt;
    OverrideRedirect: TBool;
    Colormap: TColormap;
    Cursor: TCursor
  end;
  
  { XHostAddress }
  PXHostAddress = ^TXHostAddress;
  TXHostAddress = record
    Family, Length: Integer;
    address: PChar
  end;

  { XImage }
  PXImage = ^TXImage;
  CreateImageProc = ^function (display: PDisplay; Visual: PVisual;
    Depth: Cardinal; Format: Integer; Offset: Integer; Data: PChar;
    Width, Height: Cardinal; BitmapPad, BytesPerLine: Integer): PXImage;
  DestroyImageProc = ^function (img: PXImage): Integer;
  GetPixelProc = ^function (img: PXImage; x, y: Integer): MedCard;
  PutPixelProc = ^function (img: PXImage; x, y: Integer; c: MedCard): Integer;
  SubImageProc = ^function (img: PXImage; x, y: Integer; a, b: Cardinal): PXImage;
  AddPixelProc = ^function (img: PXImage; pix: MedInt): Integer;
  TXImageFuncs = record
    CreateImage: CreateImageProc;
    DestroyImage: DestroyImageProc;
    GetPixel: GetPixelProc;  
    PutPixel: PutPixelProc;
    SubImage: SubImageProc;
    AddPixel: AddPixelProc
  end;
  TXImage = record
    Width, Height, Offset: Integer;
    Format: Integer;
    Data: PChar;
    ByteOrder, BitmapUnit, BitmapBitOrder, BitmapPad,
      Depth, BytesPerLine, BytesPerPixel: Integer;
    RedMask, GreenMask, BlueMask: MedCard;
    ObData: TXPointer;
    f: TXImageFuncs
  end;

  { XWindowChanges }
  PXWindowChanges = ^TXWindowChanges;
  TXWindowChanges = record
    X, Y, Width, Height, BorderWidth: Integer;
    Sibling: TWindow;
    StackMode: Integer
  end;

  { XColor }
  PXColor = ^TXColor;
  TXColor = record
    Pixel: MedCard;
    Red, Green, Blue: ShortCard;
    Flags, Pad: Char
  end;

  { XSegment }
  PXSegment = ^TXSegment;
  TXSegment = record
    X1, Y1, X2, Y2: Integer
  end;
  
  { XPoint }
  PXPoint = ^TXPoint;
  TXPoint = record
    X, Y: Integer
  end;
  
  { XRectangle }
  PXRectangle = ^TXRectangle;
  TXRectangle = record
    X, Y: ShortInt;
    Width, Height: ShortCard
  end;

  { XArc }
  PXArc = ^TXArc;
  TXArc = record
    X, Y: ShortInt;
    Width, Height: ShortCard;
    Angle1, Angle2: ShortInt
  end;

  { XKeyboardControl }
  PXKeyboardControl = ^TXKeyboardControl;
  TXKeyboardControl = record
    KeyClickPercent, BellPercent, BellPitch, BellDuration,
      Led, LedMode, Key, AutoRepeatMode: Integer
  end;

  { XKeyboardState }
  PXKeyboardState = ^TXKeyboardState;
  TXKeyboardState = record
    KeyClickPercent, BellPercent: Integer;
    BellPitch, BellDuration: Cardinal;
    LedMask: MedCard; 
    GlobalAutoRepeat: Integer;
    AutoRepeats: array [0 .. 31] of Char
  end;

  { XTimeCoord }
  PXTimeCoord = ^TXTimeCoord;
  TXTimeCoord = record
    time: TTime;
    X, Y: ShortInt
  end;
  
  { XModifierKeymap }
const
  ModifierArrayMax = High (Integer) div Sizeof (TKeyCode);
type
  PModifierArray = ^TModifierArray;
  TModifierArray = array [0 .. ModifierArrayMax] of TKeyCode;
  PXModifierKeymap = ^TXModifierKeymap;
  TXModifierKeymap = record
    MaxKeyPerMod: Integer;
    Modifiermap: PModifierArray 
  end;
  
  { XKeyEvent }
  PXKeyEvent = ^TXKeyEvent;
  TXKeyEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Window, Root, SubWindow: TWindow;
    time: TTime;
    X, Y, XRoot, YRoot: Integer;
    State, KeyCode: Cardinal;
    SameScreen: TBool
  end;
  
  { XKeyPressedEvent }
  PXKeyPressedEvent = ^TXKeyPressedEvent;
  TXKeyPressedEvent = TXKeyEvent;
  
  { XKeyReleasedEvent }
  PXKeyReleasedEvent = ^TXKeyReleasedEvent;
  TXKeyReleasedEvent = ^TXKeyEvent;
  
  { XButtonEvent }
  PXButtonEvent = ^TXButtonEvent;
  TXButtonEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Window, Root, SubWindow: TWindow;
    time: TTime;
    X, Y, XRoot, YRoot: Integer;
    State, Button: Cardinal;
    SameScreen: TBool
  end;
  
  { XButtonPressedEvent }
  PXButtonPressedEvent = ^TXButtonPressedEvent;
  TXButtonPressedEvent = TXButtonEvent;
  
  { XButtonReleasedEvent }
  PXButtonReleasedEvent = ^TXButtonReleasedEvent;
  TXButtonReleasedEvent = TXButtonEvent;

  { XMotionEvent }
  PXMotionEvent = ^TXMotionEvent;
  TXMotionEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Window, Root, SubWindow: TWindow;
    time: TTime;
    X, Y, XRoot, YRoot: Integer;
    State: Cardinal;
    IsHint: Char;
    SameScreen: TBool
  end;
  
  { XPointerMovedEvent }
  PXPointerMovedEvent = ^TXPointerMovedEvent;
  TXPointerMovedEvent = TXMotionEvent;

  { XCrossingEvent }
  PXCrossingEvent = ^TXCrossingEvent;
  TXCrossingEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Window, Root, SubWindow: TWindow;
    time: TTime;
    X, Y, XRoot, YRoot: Integer;
    Mode, Detail: Integer;
    SameScreen: TBool;
    Focus: TBool;
    State: Cardinal
  end;
  
  { XEnterWindowEvent }
  PXEnterWindowEvent = ^TXEnterWindowEvent;
  TXEnterWindowEvent = TXCrossingEvent;
  
  { XLeaveWindowEvent }
  PXLeaveWindowEvent = ^TXLeaveWindowEvent;
  TXLeaveWindowEvent = TXCrossingEvent;
  
  { XFocusChangeEvent }
  PXFocusChangeEvent = ^TXFocusChangeEvent;
  TXFocusChangeEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Window: TWindow;
    Mode, Detail: Integer
  end;
  
  { XFocusInEvent }
  PXFocusInEvent = ^TXFocusInEvent;
  TXFocusInEvent = TXFocusChangeEvent;
  
  { XFocusOutEvent }
  PXFocusOutEvent = ^TXFocusOutEvent;
  TXFocusOutEvent = TXFocusChangeEvent;
  
  { XKeymapEvent }
  PXKeymapEvent = ^TXKeymapEvent;
  TXKeymapEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Window: TWindow;
    KeyVector: array [0 .. 31] of Char
  end;
  
  { XExposeEvent }
  PXExposeEvent = ^TXExposeEvent;
  TXExposeEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Window: TWindow;
    X, Y, Width, Height, Count: Integer
  end;

  { XGraphicsExposeEvent }
  PXGraphicsExposeEvent = ^TXGraphicsExposeEvent;
  TXGraphicsExposeEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Drawable: TDrawable;
    X, Y, Width, Height, Count: Integer;
    MajorCode, MinorCode: Integer
  end;

  { XNoExposeEvent }
  PXNoExposeEvent = ^TXNoExposeEvent;
  TXNoExposeEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Drawable: TDrawable;
    MajorCode, MinorCode: Integer
  end;
  
  { XVisibilityEvent }
  PXVisibilityEvent = ^TXVisibilityEvent;
  TXVisibilityEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Window: TWindow;
    State: Integer
  end;

  { XCreateWindowEvent }
  PXCreateWindowEvent = ^TXCreateWindowEvent;
  TXCreateWindowEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Parent, Window: TWindow;
    X, Y, Width, Height: Integer;
    BorderWidth: Integer;
    OverrideRedirect: TBool
  end;
  
  { XDestroyWindowEvent }
  PXDestroyWindowEvent = ^TXDestroyWindowEvent;
  TXDestroyWindowEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Event, Window: TWindow
  end;

  { XUnmapEvent }
  PXUnmapEvent = ^TXUnmapEvent;
  TXUnmapEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Event, Window: TWindow;
    FromConfigure: TBool
  end;
  
  { XMapEvent }
  PXMapEvent = ^TXMapEvent;
  TXMapEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Event, Window: TWindow;
    OverrideRedirect: TBool
  end;

  { XMapRequestEvent }
  PXMapRequestEvent = ^TXMapRequestEvent;
  TXMapRequestEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Parent, Window: TWindow
  end;

  { XReparentEvent }
  PXReparentEvent = ^TXReparentEvent;
  TXReparentEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Event, Window, Parent: TWindow;
    X, Y: Integer;
    OverrideRedirect: TBool
  end;

  { XConfigureEvent }
  PXConfigureEvent = ^TXConfigureEvent;
  TXConfigureEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Event, Window: TWindow;
    X, Y, Width, Height: Integer;
    BorderWidth: Integer;
    Above: TWindow;
    OverrideRedirect: TBool
  end;

  { XGravityEvent }
  PXGravityEvent = ^TXGravityEvent;
  TXGravityEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Event, Window: TWindow;
    X, Y: Integer
  end;

  { XResizeRequestEvent }
  PXResizeRequestEvent = ^TXResizeRequestEvent;
  TXResizeRequestEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Window: TWindow;
    Width, Height: Integer
  end;
  
  { XConfigureRequestEvent }
  PXConfigureRequestEvent = ^TXConfigureRequestEvent;
  TXConfigureRequestEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Parent, Window: TWindow;
    X, Y, Width, Height: Integer;
    BorderWidth: Integer;
    Above: TWindow;
    Detail: Integer;
    ValueMask: MedCard
  end;
  
  { XCirculateEvent }
  PXCirculateEvent = ^TXCirculateEvent;
  TXCirculateEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Event, Window: TWindow;
    Place: Integer
  end;
  
  { XCirculateRequestEvent }
  PXCirculateRequestEvent = ^TXCirculateRequestEvent;
  TXCirculateRequestEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Parent, Window: TWindow;
    Place: Integer
  end;

  { XPropertyEvent }
  PXPropertyEvent = ^TXPropertyEvent;
  TXPropertyEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Window: TWindow;
    Atom: TAtom;
    Time: TTime;
    State: Integer
  end;

  { XSelectionClearEvent }
  PXSelectionClearEvent = ^TXSelectionClearEvent;
  TXSelectionClearEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Window: TWindow;
    Selection: TAtom;
    Time: TTime
  end;
  
  { XSelectionRequestEvent }
  PXSelectionRequestEvent = ^TXSelectionRequestEvent;
  TXSelectionRequestEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Owner, Requestor: TWindow;
    Selection, Target, TProperty: TAtom;  {!! Different Name }
    Time: TTime
  end;

  { XSelectionEvent }
  PXSelectionEvent = ^TXSelectionEvent;
  TXSelectionEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Requestor: TWindow;
    Selection, TTarget, TProperty: TAtom;  {!! Different Names }
    Time: TTime
  end;

  { XColormapEvent }
  PXColormapEvent = ^TXColormapEvent;
  TXColormapEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Window: TWindow;
    ANew: TBool;  {!! Different Name }
    State: Integer
  end;
  
  { XClientMessageEvent }
  TXClientMessageEventData = record
    case Byte of
      0: (b: array [0 .. 19] of Char);
      1: (s: array [0 .. 9] of ShortInt);
      2: (l: array [0 .. 4] of MedInt);
  end;
  PXClientMessageEvent = ^TXClientMessageEvent;
  TXClientMessageEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Window: TWindow;
    MessageType: TAtom;
    Format: Integer;
    Data: TXClientMessageEventData
  end;
    
  { XMappingEvent }
  PXMappingEvent = ^TXMappingEvent;
  TXMappingEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Window: TWindow;
    Request: Integer;
    FirstKeycode: Integer;
    Count: Integer
  end;

  { XErrorEvent }
  PXErrorEvent = ^TXErrorEvent;
  TXErrorEvent = record
    EventType: Integer;  {!! Different Name }
    Display: PDisplay;
    ResourceId: TXID;
    Serial: MedCard;
    ErrorCode, RequestCode, MinorCode: Byte
  end;
  
  { XAnyEvent }
  PXAnyEvent = ^TXAnyEvent;
  TXAnyEvent = record
    EventType: Integer;  {!! Different Name }
    Serial: MedCard;
    SendEvent: TBool;
    Display: PDisplay;
    Window: TWindow
  end;
    
  { XEvent }
  PXEvent = ^TXEvent;
  TXEvent = record
    case Byte of
       1: (EventType: Integer);  { Must not be changed, First Element }
       2: (XAny: TXAnyEvent);
       3: (XKey: TXKeyEvent);
       4: (XButton: TXButtonEvent);
       5: (XMotion: TXMotionEvent);
       6: (XCrossing: TXCrossingEvent);
       7: (XFocus: TXFocusChangeEvent);
       8: (XExpose: TXExposeEvent);
       9: (XGraphicsExpose: TXExposeEvent);
      10: (XNoExpose: TXNoExposeEvent);
      11: (XVisibility: TXVisibilityEvent);
      12: (XCreateWindow: TXCreateWindowEvent);
      13: (XDestroyWindow: TXDestroyWindowEvent);
      14: (XUnMap: TXUnmapEvent);
      15: (XMap: TXMapEvent);
      16: (XMapRequest: TXMapRequestEvent);
      17: (XReparent: TXReparentEvent);
      18: (XConfigure: TXConfigureEvent);
      19: (XGravity: TXGravityEvent);
      20: (XResizeRequest: TXResizeRequestEvent);
      21: (XConfigureRequest: TXConfigureRequestEvent);
      22: (XCirculate: TXCirculateEvent);
      23: (XCirculateRequest: TXCirculateRequestEvent);
      24: (XProperty: TXPropertyEvent);
      25: (XSelectionClear: TXSelectionClearEvent);
      26: (XSelectionRequest: TXSelectionRequestEvent);
      27: (XSelection: TXSelectionEvent);
      28: (XColormap: TXColormapEvent);
      29: (XClient: TXClientMessageEvent);
      30: (XMapping: TXMappingEvent);
      31: (XError: TXErrorEvent);
      32: (XKeymap: TXKeymapEvent);
      33: (pad: array [0 .. 23] of MedInt)
  end;

  { XCharStruct }
  PXCharStruct = ^TXCharStruct;
  TXCharStruct = record
    lBearing, rBearing, Width, Ascend, Descend: ShortInt;
    Attributes: ShortCard
  end;
  
  { XFontProp }
  PXFontProp = ^TXFontProp;
  TXFontProp = record
    Name: TAtom;
    Card32: MedCard  {!! Silly Name!!}
  end;
  
  { XFontStruct }
const
  MaxPropertiesArray = High (Integer) div Sizeof (TXFontProp);
type
  PPropertiesArray = ^TPropertiesArray;
  TPropertiesArray = array [0 .. MaxPropertiesArray] of TXFontProp;
  { @@ Should be an array?? }
  PPXFontStruct = ^PXFontStruct;
  PXFontStruct = ^TXFontStruct;
  TXFontStruct = record
    ExtData: PXExtData;
    FID: TFont;
    Direction, MinCharOrByte2, MaxCharOrByte2: Cardinal;
    MinByte1, MaxByte1: Cardinal;
    AllCharsExists: TBool;
    DefaultChar: Cardinal;
    NProperties: Integer;
    Properties: PPropertiesArray;  
    MinBounds, MaxBounds: TXCharStruct;
    PerChar: PXCharStruct;  { @@ Don't know what it is:-( }
    Ascend, Descent: Integer
  end;

  { XTextItem }
  PXTextItem = ^TXTextItem;
  TXTextItem = record
    Chars: PChar;  {!! Pointer to String }
    NChars, Delta: Integer;
    Font: TFont
  end;
  
  { XChar2b }
  PXChar2b = ^TXChar2b;
  TXChar2b = record
    Byte1, Byte2: Byte
  end;
  
  { XTextItem16 }
  PXTextItem16 = ^TXTextItem16;
  TXTextItem16 = record
    Chars: PXChar2B;  {!! Pointer to 2Byte Character }
    NChars, Delta: Integer;
    Font: TFont
  end;
  
  { XEDataObject }
  PXEDataObject = ^TXEDataObject;
  TXEDataObject = record
    case Byte of
      1: (Display: PDisplay);
      2: (gc: PGC);
      3: (Visual: PVisual);
      4: (Screen: PScreen);
      5: (PixmapFormat: PScreenFormat);
      6: (Font: PXFontStruct)
  end;

  { XFontSetExtents }
  PXFontSetExtents = ^TXFontSetExtents;
  TXFontSetExtents = record
    MaxIncExtent, MaxLogicalExtent: TXRectangle
  end;
  
  { XOM }
  PXOM = Pointer;  { Output Method }
  
  { XOC }
  PXOC = Pointer;  { Output Context }
  
  { XFontSet }
  PXFontSet = Pointer;
  
  { XmbTextItem }
  PXmbTextItem = ^TXmbTextItem;
  TXmbTextItem = record
    Chars: PChar;
    NChars, Delta: Integer;
    FontSet: PXFontSet
  end;

  { XwcTextItem }
  PXwcTextItem = ^TXwcTextItem;
  TXwcTextItem = record
    Chars: PWCharT;  {!! Array of Wide Chars (kind of String) }
    NChars, Delta: Integer;
    FontSet: PXFontSet
  end;

  { XOMCharSetList }
const
  MaxCharsetList = High (Integer) div SizeOf (PChar);
type
  PCharsetListArray = ^TCharsetListArray;
  TCharsetListArray = array [0 .. MaxCharsetList] of PChar;
  PXOMCharSetList = ^TXOMCharSetList;
  TXOMCharSetList = record
    CharsetCount: Integer;
    CharsetList: PCharsetListArray
  end;

  { XOrientation, see XOMOrientation_* in consts section }
  PXOrientation =^TXOrientation;
  TXOrientation = Integer;

  { XOMOrientation }
  PXOMOrientation = ^TXOMOrientation;
  TXOMOrientation = record
    NumOrientation: Integer;
    Orientation: PXOrientation
  end;

  { XOMFontInfo }
const
  MaxFontStructList = High (Integer) div Sizeof (PXFontStruct);
  MaxFontnameList = High (Integer) div SizeOf (PChar);
type
  PFontStructListArray = ^TFontStructListArray;
  TFontStructListArray = array [0 .. MaxFontStructList] of PXFontStruct;
  PFontnameListArray = ^TFontnameListArray;
  TFontnameListArray = array [0 .. MaxFontnameList] of PChar;
  PXOMFontInfo = ^TXOMFontInfo;
  TXOMFontInfo = record
    NumFont: Integer;
    FontStructList: PFontStructListArray;
    FontNameList: PFontnameListArray
  end;

  { XIM }
  PXIM = Pointer;  { Input Method }
  
  { XIC }
  PXIC = Pointer;  { Input Context }

  PXIMProc = ^procedure (a: PXIM; b, c: TXPointer);
  PXICProc = ^procedure (a: PXIC; b, c: TXPointer);
  PXIDProc = ^procedure (a: PDisplay; b, c: TXPointer);
  PXVaNestedList = ^function: TXPointer;  {!! I'm not sure...}

  { XIMStyles }
const
  MaxSupportedStyles = High (Integer) div SizeOf (TXIMStyle);
type
  PSupportedStylesArray = ^TSupportedStylesArray;
  TSupportedStylesArray = array [0 .. MaxSupportedStyles] of TXIMStyle;
  PXIMStyles = ^TXIMStyles;
  TXIMStyles = record
    CountStyles: ShortCard;
    SupportedStyles: PSupportedStylesArray
  end;

  { XIMCallback }
  PXIMCallback = ^TXIMCallback;
  TXIMCallback = record
    ClientData: TXPointer;
    Callback: PXIMProc
  end;

  { XICCallback }
  PXICCallback = ^TXICCallback;
  TXICCallback = record
    ClientData: TXPointer;
    Callback: PXICProc
  end;

  { XIMText }
  TXIMText_helper = record
    case Byte of
      0: (MultiByte: PChar);
      1: (WideChar: PWCharT);
  end;
  PXIMText = ^TXIMText;
  TXIMText = record
    Length: ShortCard;
    FeedBack: PXIMFeedback;
    EncodingIsWChar: TBool;
    AString: TXIMText_helper  {!! different name}
  end;

  { XIMPreeditStateNotifyCallbackStruct }
  PXIMPreeditStateNotifyCallbackStruct = ^TXIMPreeditStateNotifyCallbackStruct;
  TXIMPreeditStateNotifyCallbackStruct = record
    State: TXIMPreeditState
  end;

  { XIMStringConversionText }
  TXIMStringConversionText_helper = record
    case Byte of
      0: (mbs: PChar);
      1: (wcs: PWCharT);
  end;
  PXIMStringConversionText = ^TXIMStringConversionText;
  TXIMStringConversionText = record
    Length: ShortCard;
    Feedback: PXIMStringConversionFeedback;
    EncodingIsWChar: TBool;
    AString: TXIMStringConversionText_helper  {!! Different name}
  end;

  { XIMCaretDirection, see XIM* in consts section above }
  PXIMCaretDirection = ^TXIMCaretDirection;
  TXIMCaretDirection = Integer;

  { XIMStringConversionCallbackStruct }
  PXIMStringConversionCallbackStruct = ^TXIMStringConversionCallbackStruct;
  TXIMStringConversionCallbackStruct = record
    Position: TXIMStringConversionPosition;
    Direction: TXIMCaretDirection;
    Operation: TXIMStringConversionOperation;
    Factor: ShortCard;
    Text: PXIMStringConversionText
  end;

  { XIMPreeditDrawCallbackStruct }
  PXIMPreeditDrawCallbackStruct = ^TXIMPreeditDrawCallbackStruct;
  TXIMPreeditDrawCallbackStruct = record
    Caret, ChgFirst, ChgLength: Integer;
    Text: PXIMText
  end;

  { XIMCaretStyle, See XIMIs* constants above }
  TXIMCaretStyle = Integer;

  { XIMPreeditCaretCallbackStruct }
  TXIMPreeditCaretCallbackStruct = record
    Position: Integer;
    Direction: TXIMCaretDirection;
    Style: TXIMCaretStyle
  end;

  { XIMStatusDataType, See XIM*Type constants above }
  TXIMStatusDataType = Integer;
  
  TXIMStatusDrawCallbackStruct_helper = record
    case Byte of
      0: (Text: PXIMText);
      1: (Bitmap: TPixmap);
  end;
  TXIMStatusDrawCallbackStruct = record
    DataType: TXIMStatusDataType;  {!! Different name}
    Data: TXIMStatusDrawCallbackStruct_helper
  end;  

  PXIMHotKeyTrigger = ^TXIMHotKeyTrigger;
  TXIMHotKeyTrigger = record
    KeySym: TKeySym;
    Modfier, ModifierMask: Integer
  end;
  
  { XIMHotKeyTriggers }
const
  MaxKeysInKeyArray = High (Integer) div SizeOf (TXIMHotKeyTrigger);
type
  PKeysInKeyArray = ^TKeysInKeyArray;
  TKeysInKeyArray = array [0 .. MaxKeysInKeyArray] of TXIMHotKeyTrigger;
  TXIMHotKeyTriggers = record
    NumHotKey: Integer;
    Key: PKeysInKeyArray
  end;

  { XIMValuesList }
const
  MaxSupportedValues = High (Integer) div SizeOf (PChar);
type
  PSupportedValuesArray = ^TSupportedValuesArray;
  TSupportedValuesArray = array [0 .. MaxSupportedValues] of PChar;
  TXIMValuesList = record
    CountValues: ShortCard;
    SupportedValues: PSupportedValuesArray
  end;

{@section Makros from Xlib.h}

function ConnectionNumber (dpy: PDisplay): Integer;
asmname '_p_ConnectionNumber';

function RootWindow (dpy: PDisplay; Scr: Integer): TWindow;
asmname '_p_RootWindow';

function DefaultScreen (dpy: PDisplay): Integer;
asmname '_p_DefaultScreen';

function DefaultRootWindow (dpy: PDisplay): TWindow;
asmname '_p_DefaultRootWindow';

function DefaultVisual (dpy: PDisplay; Scr: Integer): PVisual;
asmname '_p_DefaultVisual';

function DefaultGC (dpy: PDisplay; Scr: Integer): PGC;
asmname '_p_DefaultGC';

function BlackPixel (dpy: PDisplay; Scr: Integer): MedCard;
asmname '_p_BlackPixel';

function WhitePixel (dpy: PDisplay; Scr: Integer): MedCard;
asmname '_p_WhitePixel';

function AllPlanes: MedCard;
asmname '_p_AllPlanes';

function QLength (dpy: PDisplay): Integer;
asmname '_p_QLength';

function DisplayWidth (dpy: PDisplay; Scr: Integer): Integer;
asmname '_p_DisplayWidth';

function DisplayHeight (dpy:PDisplay; Scr:Integer): Integer;
asmname '_p_DisplayHeight';

function DisplayWidthMM (dpy: PDisplay; Scr: Integer): Integer;
asmname '_p_DisplayWidthMM';

function DisplayHeightMM (dpy: PDisplay; Scr: Integer): Integer;
asmname '_p_DisplayHeightMM';

function DisplayPlanes (dpy: PDisplay; Scr: Integer): Integer;
asmname '_p_DisplayPlanes';

function DisplayCells (dpy: PDisplay; Scr: Integer): Integer;
asmname '_p_DisplayCells';

function ScreenCount (dpy: PDisplay): Integer;
asmname '_p_ScreenCount';

function ServerVendor(dpy: PDisplay): PChar;
asmname '_p_ServerVendor';

function ProtocolVersion (dpy: PDisplay): Integer;
asmname '_p_ProtocolVersion';

function ProtocolRevision (dpy: PDisplay): Integer;
asmname '_p_ProtocolRevision';

function VendorRelease (dpy: PDisplay): Integer;
asmname '_p_VendorRelease';

function DisplayString(dpy: PDisplay): PChar;
asmname '_p_DisplayString';

function DefaultDepth (dpy: PDisplay; Scr: Integer): Integer;
asmname '_p_DefaultDepth';

function DefaultColormap (dpy: PDisplay; Scr: Integer): TColormap;
asmname '_p_DefaultColormap';

function BitmapUnit (dpy: PDisplay): Integer;
asmname '_p_BitmapUnit';

function BitmapBitOrder (dpy: PDisplay): Integer;
asmname '_p_BitmapBitOrder';

function BitmapPad (dpy: PDisplay): Integer;
asmname '_p_BitmapPad';

function ImageByteOrder (dpy: PDisplay): Integer;
asmname '_p_ImageByteOrder';

function NextRequest (dpy: PDisplay): MedCard;
asmname '_p_NextRequest';

function LastKnownRequestProcessed (dpy: PDisplay): MedCard;
asmname '_p_LastKnownRequestProcessed';

{ Toolkit Makros from Xlib.h }

function ScreenOfDisplay (dpy: PDisplay; Scr: Integer): PScreen;
asmname '_p_ScreenOfDisplay';

function DefaultScreenOfDisplay (dpy: PDisplay): PScreen;
asmname '_p_DefaultScreenOfDisplay';

function DisplayOfScreen  (s: PScreen): PDisplay;
asmname '_p_DisplayOfScreen';

function RootWindowOfScreen  (s: PScreen): TWindow;
asmname '_p_RootWindowOfScreen';

function BlackPixelOfScreen  (s: PScreen): MedCard;
asmname '_p_BlackPixelOfScreen';

function WhitePixelOfScreen  (s: PScreen): MedCard;
asmname '_p_WhitePixelOfScreen';

function DefaultColormapOfScreen  (s: PScreen): TColormap;
asmname '_p_DefaultColormapOfScreen';

function DefaultDepthOfScreen  (s: PScreen): Integer;
asmname '_p_DefaultDepthOfScreen';

function DefaultGCOfScreen  (s: PScreen): PGC;
asmname '_p_DefaultGCOfScreen';

function DefaultVisualOfScreen  (s: PScreen): PVisual;
asmname '_p_DefaultVisualOfScreen';

function WidthOfScreen  (s: PScreen): Integer;
asmname '_p_WidthOfScreen';

function HeightOfScreen  (s: PScreen): Integer;
asmname '_p_HeightOfScreen';

function WidthMMOfScreen  (s: PScreen): Integer;
asmname '_p_WidthMMOfScreen';

function HeightMMOfScreen  (s: PScreen): Integer;
asmname '_p_HeightMMOfScreen';

function PlanesOfScreen  (s: PScreen): Integer;
asmname '_p_PlanesOfScreen';

function CellsOfScreen  (s: PScreen): Integer;
asmname '_p_CellsOfScreen';

function MinCmapsOfScreen  (s: PScreen): Integer;
asmname '_p_MinCmapsOfScreen';

function MaxCmapsOfScreen  (s: PScreen): Integer;
asmname '_p_MaxCmapsOfScreen';

function DoesSaveUnders  (s: PScreen): TBool;
asmname '_p_DoesSaveUnders';

function DoesBackingStore  (s: PScreen): Integer;
asmname '_p_DoesBackingStore';

function EventMaskOfScreen  (s: PScreen): MedInt;
asmname '_p_EventMaskOfScreen';

function XAllocID (dpy: PDisplay): TXID;
asmname '_p_XAllocID';


{@section Prototypes from Xlib.h}

function XVaCreateNestedList (unused: Integer; ...): PXVaNestedList;
asmname 'XVaCreateNestedList';

function XLoadQueryFont (display: PDisplay; name: PChar): PXFontStruct;
asmname 'XLoadQueryFont';

function XQueryFont (display: PDisplay; FontID: TXID): PXFontStruct;
asmname 'XQueryFont';

function XGetMotionEvents (display: PDisplay; Win: TWindow; 
  start, stop: TTIme; var NEventsReturn: Integer): PXTimeCoord;
asmname 'XGetMotionEvents';

function XDeleteModifiermapEntry (modmap: PXModifierKeymap;
  KeycodeEntry: TKeyCode; Modifier: Integer): PXModifierKeymap;
asmname 'XDeleteModifiermapEntry';

function XGetModifierMapping (display: PDisplay): PXModifierKeymap;
asmname 'XGetModifierMapping';

function XInsertModifiermapEntry (modmap: PXModifierKeymap;
  KeycodeEntry: TKeyCode; Modifier: Integer): PXModifierKeymap;
asmname 'XInsertModifiermapEntry';

function XNewModifiermap (MaxKeysPerMod: Integer): PXModifierKeymap;
asmname 'XNewModifiermap';

function XCreateImage (Display: PDisplay; Visual: PVisual;
  Depth: MedCard; Format, Offset: Integer; Data: PChar;
  Width, Height: Cardinal; BitmapPad, BytesPerLine: Integer): PXImage;
asmname 'XCreateImage';

function XInitImage (Image: PXImage): TStatus;
asmname 'XInitImage';

function XGetImage (display: PDisplay; draw: TDrawable; x, y: Integer;
  Width, Height: Cardinal; PlaneMask: MedCard; Format: Integer)
 : PXImage;
asmname 'XGetImage';

function XGetSubImage (display: PDisplay; draw: TDrawable; x, y: Integer;
  Width, Height: Cardinal; PlaneMask: MedCard; Format: Integer;
  DestImage: PXImage; DestX, DestY: Integer): PXImage;
asmname 'XGetSubImage';

function XOpenDisplay (Name: PChar): PDisplay;
asmname 'XOpenDisplay';

procedure XrmInitialize;
asmname 'XrmInitialize';

function XFetchBytes (display: PDisplay; var NBytesRet: Integer): PChar;
asmname 'XFetchBytes';

function XFetchBuffer (display: PDisplay; var NBytesRet: Integer;
  Buffer: Integer): PChar;
asmname 'XFetchBuffer';

function XGetAtomName (display: PDisplay; Atom: TAtom): PChar;
asmname 'XGetAtomName';

const
  AtomsArrayMax = High (Integer) div SizeOf (TAtom);
type
  PAtomsArray = ^TAtomsArray;
  TAtomsArray = array [0 .. AtomsArrayMax] of TAtom;
  
function XGetAtomNames (display: PDisplay; Atoms: PAtomsArray; Count: Integer;
  var NamesRet: PChar): TStatus;
asmname 'XGetAtomNames';

function XGetDefault (display: PDisplay; ProgramName, Option: PChar): PChar;
asmname 'XGetDefault';

function XDisplayName (aSTring: PChar): PChar;
asmname 'XDisplayName';

function XKeysymToString (keysym: TKeySym): PChar;
asmname 'XKeysymToString';

function XSynchronize (dpy: PDisplay; OnOff: TBool): Integer;
asmname 'XSynchronize';

type
  { For XSetAfterFunction () }
  PSetAfterFunction = ^function (dpy: PDisplay): Integer;

function XSetAfterFunction (dpy: PDisplay; proc: PSetAfterFunction): Integer;
asmname 'XSetAfterFunction';

function XInternAtom (dpy: PDisplay; AtomName: PChar; OnlyIfExists: TBool): TAtom;
asmname 'Atom';

{ We bought the type "PCharsetListArray" from another type. It's just an
array of Strings in C }
function XInternAtoms (dpy: PDisplay; Names: PCharsetListArray; 
  Count: Integer; OnlyIfExists: TBool; AtomsRet: PAtomsArray): TStatus;
asmname 'XInternAtoms';

function XCopyColormapAndFree (dpy: PDisplay; ColorMap: TColorMap): TColormap;
asmname 'XCopyColormapAndFree';

function XCreateColormap (dpy: PDisplay; W: TWindow; visual: PVisual;
  alloc: Integer): TColormap;
asmname 'XCreateColormap';

function XCreatePixmapCursor (dpy: PDisplay; Source, Mask: TPixmap;
  FgColor, BgColor: PXColor; X, Y: Cardinal): TCursor;
asmname 'XCreatePixmapCursor';

function XCreateGlyphCursor (dpy: PDisplay; SourceFont, MaskFont: TFont;
  SourceChar, MaskChar: Cardinal; FgColor, BgColor: PXColor): TCursor;
asmname 'XCreateGlyphCursor';

function XCreateFontCursor (dpy: PDisplay; Shape: Cardinal): TCursor;
asmname 'XCreateFontCursor';

function XLoadFont (dpy: PDisplay; Name: PChar): TFont;
asmname 'XLoadFont';

{ Valuemask is one of GCFunction, GCPlaneMask, GCForeground, 
(and much more) }
function XCreateGC (dpy: PDisplay; D: TDrawable; ValueMask: MedCard;
  Values: PXGcValues): PGc;
asmname 'XCreateGC';

function XGContextFromGC (gc: PGc): TGContext;
asmname 'XGContextFromGC';

procedure XFlushGC (dpy: PDisplay; gc: PGc);
asmname 'XFlushGC';

function XCreatePixmap (dpy: PDisplay; D: TDrawable;
  Width, Height, Depth: Cardinal): TPixmap;
asmname 'XCreatePixmap';

function XCreateBitmapFromData (dpy: PDisplay; D: TDrawable; Data: PChar;
  Width, Height: Cardinal): TPixmap;
asmname 'XCreateBitmapFromData';

function XCreatePixmapFromBitmapData (dpy: PDisplay; D: TDrawable; 
  Data: PChar; Width, Height: Cardinal; Fg, Bg: MedCard;
  Depth: Cardinal): TPixmap;
asmname 'XCreatePixmapFromBitmapData';

function XCreateSimpleWindow (dpy: PDisplay; window: TWindow; x, y: Integer;
  Width, Height, BorderWidth: Cardinal; Border, Background: MedCard): TWindow;
asmname 'XCreateSimpleWindow';

function XGetSelectionOwner (dpy: PDisplay; atom: TAtom): TWindow;
asmname 'XGetSelectionOwner';

function XCreateWindow  (dpy: PDisplay; window: TWindow; x, y: Integer;
  Width, Height, BorderWidth: Cardinal; Depth: Integer; Klass: Cardinal;
  visual: PVisual; ValueMask: MedCard;  Attribs: PXSetWindowAttributes): TWindow;
asmname 'XCreateWindow';

const
  ColormapArrayMax = High (Integer) div SizeOf (TColorMap);
type
  PColormapArray = ^TColormapArray;
  TColormapArray = array [0 .. ColormapArrayMax] of TColormap;
function XListInstalledColormaps (dpy: PDisplay; W: TWindow;
  var NumRet: Integer): PColormapArray;
asmname 'XListInstalledColormaps';

{ We bought the type "PCharsetListArray" from another type. It's just an
array of Strings in C }
function XListFonts (dpy: PDisplay; Pattern: PChar; MaxNames: Integer;
  var ActualCountReturn: Integer): PCharsetListArray;
asmname 'XListFonts';

function XListFontsWithInfo (dpy: PDisplay; Pattern: PChar; MaxNames: Integer;
  var CountReturn: Integer; InfoReturn: PFontStructListArray): PCharsetListArray;
asmname 'XListFontsWithInfo';

function XGetFontPath (dpy: PDisplay; var NPathsRet: Integer):PCharsetListArray;
asmname 'XGetFontPath';

function XListExtensions (dpy: PDisplay; var NExtensionsRet: Integer): PCharsetListArray;
asmname 'XListExtensions';

function XListProperties (dpy: PDisplay; W: TWindow; var NumPropRet: Integer): PAtomsArray;
asmname 'XListProperties';

const
  XHostAddressArrayMax = High (Integer) div SizeOf (TXHostAddress);
type
  PXHostAddressArray = ^TXHostAddressArray;
  TXHostAddressArray = array [0 .. XHostAddressArrayMax] of TXHostAddress;
function XListHosts (dpy: PDisplay; var NHostsRet: Integer; var StateRet:
TBool): PXHostAddressArray;
asmname 'XListHosts';

function XKeycodeToKeysym (dpy: PDisplay; KeyCode: TKeyCode; Index: Integer): TKeySym;
asmname 'XKeycodeToKeysym';

{ @@ Name clash with constant "XLookupKeySym" above}
{!! Different Name}
function XLookupKeysymP (dpy: PDisplay; KeyEvent: PXKeyEvent; Index: Integer): TKeySym;
asmname 'XLookupKeysym';

function XGetKeyboardMapping (dpy: PDisplay; FirstKeyCode: TKeyCode;
  KeyCodeCount: Integer; var KeysymsPerKeycodeRet: Integer): TKeySym;
asmname 'XGetKeyboardMapping';

function XStringToKeysym (aString: PChar): TKeySym;
asmname 'XStringToKeysym';

function XMaxRequestSize (dpy: PDisplay): MedInt;
asmname 'XMaxRequestSize';

function XExtendedMaxRequestSize (dpy: PDisplay): MedInt;
asmname 'XExtendedMaxRequestSize';

function XResourceManagerStringfunction (dpy: PDisplay): PChar;
asmname 'XResourceManagerString';

function XScreenResourceString (screen: PScreen): PChar;
asmname 'XScreenResourceString';

function XDisplayMotionBufferSize (dpy: PDisplay): MedCard;
asmname 'XDisplayMotionBufferSize';

function XVisualIDFromVisual (visual: PVisual): TVisualID;
asmname 'XVisualIDFromVisual';

{ Multithreading Routines }

function XInitThreads: TStatus;
asmname 'XInitThreads';

procedure XLockDisplay (dpy: PDisplay);
asmname 'XLockDisplay';

procedure XUnlockDisplay (dpy: PDisplay);
asmname 'XUnlockDisplay';

{ routines for dealing with extensions }

function XInitExtension (dpy: PDisplay; Name: PChar): PXExtCodes;
asmname 'XInitExtension';

function XAddExtension (dpy: PDisplay): PXExtCodes;
asmname 'XAddExtension';

const
  XExtDataArrayMax = High (Integer) div SizeOf (TXExtData);
type
  PXExtDataArray = ^TXExtDataArray;
  TXExtDataArray = array [0 .. XExtDataArrayMax] of TXExtData;
function XFindOnExtensionList (Structure: PXExtDataArray; Number: Integer): PXExtData;
asmname 'XFindOnExtensionList';

function XEHeadOfExtensionList (obj: TXEDataObject): PXExtDataArray;
asmname 'XEHeadOfExtensionList';

{ Routines, which also have a C-Macro }

function XRootWindow (dpy: PDisplay; ScreenNum: Integer): TWindow;
asmname 'XRootWindow';

function XDefaultRootWindow (dpy: PDisplay): TWindow;
asmname 'XDefaultRootWindow';

function XRootWindowOfScreen (screen: PScreen): TWindow;
asmname 'XRootWindowOfScreen';

function XDefaultVisual (dpy: PDisplay; ScreenNum: Integer): PVisual;
asmname 'XDefaultVisual';

function XDefaultVisualOfScreen (screen: PScreen): PVisual;
asmname 'XDefaultVisualOfScreen';

function XDefaultGC (dpy: PDisplay; ScreenNum: Integer): PGc;
asmname 'XDefaultGC';

function XDefaultGCOfScreen (screen: PScreen): PGc;
asmname 'XDefaultGCOfScreen';

function XBlackPixel (dpy: PDisplay; ScreenNum: Integer): MedCard;
asmname 'XBlackPixel';

function XWhitePixel (dpy: PDisplay; ScreenNum: Integer): MedCard;
asmname 'XWhitePixel';

function XAllPlanes : MedCard;
asmname 'XAllPlanes';

function XBlackPixelOfScreen (screen: PScreen): MedCard;
asmname 'XBlackPixelOfScreen';

function XWhitePixelOfScreen (screen: PScreen): MedCard;
asmname 'XWhitePixelOfScreen';

function XNextRequest (dpy: PDisplay): MedCard;
asmname 'XNextRequest';

function XLastKnownRequestProcessed (dpy: PDisplay): MedCard;
asmname 'XLastKnownRequestProcessed';

function XServerVendor (dpy: PDisplay): PChar;
asmname 'XServerVendor';

function XDisplayString (dpy: PDisplay): PChar;
asmname 'XDisplayString';

function XDefaultColormap (dpy: PDisplay; ScrrenNumber: Integer): TColormap;
asmname 'XDefaultColormap';

function XDefaultColormapOfScreen (screen: PScreen): TColormap;
asmname 'XDefaultColormapOfScreen';

function XDisplayOfScreen (screen: PScreen): PDisplay;
asmname 'XDisplayOfScreen';

function XScreenOfDisplay (dpy: PDisplay; ScreenNumber: Integer): PScreen;
asmname 'XScreenOfDisplay';

function XDefaultScreenOfDisplay (dpy: PDisplay): PScreen;
asmname 'XDefaultScreenOfDisplay';

function XEventMaskOfScreen (screen: PScreen): MedInt;
asmname 'XEventMaskOfScreen';

function XScreenNumberOfScreen (screen: PScreen): Integer;
asmname 'XScreenNumberOfScreen';

type
  PXErrorHandler = ^function (dpy: PDisplay; ErrEvent: PXErrorEvent): Integer;

function XSetErrorHandler (handler: PXErrorHandler): PXErrorHandler;
asmname 'XSetErrorHandler';

type
  PXIOErrorHandler = ^function (dpy: PDisplay): Integer;

function XSetIOErrorHandler (handler: PXIOErrorHandler): PXIOErrorHandler;
asmname 'XSetIOErrorHandler';

const
  XPixmapFormatValuesArrayMax = High (Integer) div SizeOf (TXExtData);
type
  PXPixmapFormatValuesArray = ^TXPixmapFormatValuesArray;
  TXPixmapFormatValuesArray = array [0 .. XPixmapFormatValuesArrayMax] of TXPixmapFormatValues;

function XListPixmapFormats (dpy: PDisplay; var CountRet: Integer):
  PXPixmapFormatValuesArray;
asmname 'XListPixmapFormats';

const
  IntegerArrayMax = High (Integer) div SizeOf (Integer);
type
  PIntegerArray = ^TIntegerArray;
  TIntegerArray = array [0 .. IntegerArrayMax] of Integer;

function XListDepths (dpy: PDisplay; ScreenNumber: Integer;
  var CountRet: Integer): PIntegerArray;
asmname 'XListDepths';

function XReconfigureWMWindow (dpy: PDisplay; W: TWindow; ScreenNumber: Integer;
  Mask: Cardinal; Changes: PXWindowChanges): TSTatus;
asmname 'XReconfigureWMWindow';

function XGetWMProtocols (dpy: PDisplay; w: TWindow; var atoms: PAtomsArray;
  var CountRet: Integer): TStatus;
asmname 'XGetWMProtocols';

function XSetWMProtocols (dpy: PDisplay; W: TWindow; atoms: PAtomsArray;
  Count: Integer): TSTatus;
asmname 'XSetWMProtocols';

function XIconifyWindow (dpy: PDisplay; w: TWindow; ScreenNum: Integer): TStatus;
asmname 'XIconifyWindow';

function XWithdrawWindow (dpy: PDisplay; w: TWindow; ScreenNum: Integer): TStatus;
asmname 'XWithdrawWindow';

function XGetCommand (dpy: PDisplay; w: TWindow; 
  var ArgVRet: PCharsetListArray; var ArgC: Integer): TStatus;
asmname 'XGetCommand';

const
  WindowsArrayMax = High (Integer) div SizeOf (TWindow);
type
  PWindowsArray = ^TWindowsArray;
  TWindowsArray = array [0 .. WindowsArrayMax] of TWindow;

function XGetWMColormapWindows (dpy: PDisplay; w: TWindow;
  var WindowsRet: PWindowsArray; var CountRet: Integer): TStatus;
asmname 'XGetWMColormapWindows';

function XSetWMColormapWindows (dpy: PDisplay; w: TWindow;
  ColormapWindows: PWindowsArray; Count: Integer): TStatus;
asmname 'XSetWMColormapWindows';

procedure XFreeStringList (list: PCharsetListArray);
asmname 'XFreeStringList';

function XSetTransientForHint (dpy: PDisplay; W, PropWindow: TWindow): Integer;
asmname 'XSetTransientForHint';

{ The following are given in alphabetical order }

function XActivateScreenSaver (dpy: PDisplay): Integer;
asmname 'XActivateScreenSaver';

function XAddHost (dpy: PDisplay; var Hosts: TXHostAddress): Integer;
asmname 'XAddHost';

function XAddHosts (dpy: PDisplay; Hosts: PXHostAddressArray;
  NumHosts: Integer): Integer;
asmname 'XAddHosts';

{ @@ I'm not sure with the parameters}
function XAddToExtensionList (structure: PXExtDataArray; Data: PXExtData): Integer;
asmname 'XAddToExtensionList';

function XAddToSaveSet (dpy: PDisplay; w: TWindow): Integer;
asmname 'XAddToSaveSet';

function XAllocColor (dpy: PDisplay; colormap: TColormap;
  var ScreenInOut: TXColor): TStatus;
asmname 'XAllocColor';

function XAllocColorCells (dpy: PDisplay; ColMap: TColormap; Contig: TBool;
  var PlaneMaskRet: MedCard; NPlanes: Cardinal; var PixelRet: MedCard;
  NPixels: Cardinal): TStatus;
asmname 'XAllocColorCells';

function XAllocColorPlanes (dpy: PDisplay; ColMap: TColormap; Contig: TBool;
  var PixelsRet: MedCard; NColors, NReds, NGreens, NBlues: Integer;
  var RMaskRet: MedCard; var GMaskRet: MedCard; BMaskRet: MedCard): TStatus;
asmname 'XAllocColorPlanes';

function XAllocNamedColor (dpy: PDisplay; ColMap: TColormap;
  ColorName: PChar; var ScreenDefRet: TXColor; ExactDefRet: TXColor): TStatus;
asmname 'XAllocNamedColor';

function XAllowEvents (dpy: PDisplay; EventMode: Integer; time: TTime): Integer;
asmname 'XAllowEvents';

function XAutoRepeatOff (dpy: PDisplay): Integer;
asmname 'XAutoRepeatOff';

function XAutoRepeatOn (dpy: PDisplay): Integer;
asmname 'XAutoRepeatOn';

function XBell (dpy: PDisplay; Percent: Integer): Integer;
asmname 'XBell';

function XBitmapBitOrder (dpy: PDisplay): Integer;
asmname 'XBitmapBitOrder';

function XBitmapPad (dpy: PDisplay): Integer;
asmname 'XBitmapPad';

function XBitmapUnit (dpy: PDisplay): Integer;
asmname 'XBitmapUnit';

function XCellsOfScreen (screen: PScreen): Integer;
asmname 'XCellsOfScreen';

function XChangeActivePointerGrab (dpy: PDisplay; EventMask: Cardinal;
  cursor: TCursor; time: TTime): Integer;
asmname 'XChangeActivePointerGrab';

function XChangeGC (dpy: PDisplay; gc: PGC; ValueMask: MedCard;
  var Values: TXGCValues): Integer;
asmname 'XChangeGC';

function XChangeKeyboardControl (dpy: PDisplay; ValueMask: MedCard;
  var Values: TXKeyboardControl): Integer;
asmname 'XChangeKeyboardControl';

const
  KeysymsArrayMax = High (Integer) div SizeOf (TKeySym);
type
  PKeysymsArray = ^TKeysymsArray;
  TKeysymsArray = array [0 .. KeysymsArrayMax] of TKeysym;

function XChangeKeyboardMapping (dpy: PDisplay; FirstKeycode,
  KeysymsPerKeycode: Integer; KeySyms: PKeysymsArray;
  NumCodes: Integer): Integer;
asmname 'XChangeKeyboardMapping';

function XChangePointerControl (dpy: PDisplay; DoAccel, DoTreshold: TBool;
  AccelNumerator, AccelDenominator, Treshold: Integer): Integer;
asmname 'XChangePointerControl';

function XChangeProperty (dpy: PDisplay; w: TWindow; Propert, aType: TAtom;
  Format, Mode: Integer; Data: PChar; NElements: Integer): Integer;
asmname 'XChangeProperty';

function XChangeSaveSet (dpy: PDisplay; w: TWindow; ChangeMode: Integer): Integer;
asmname 'XChangeSaveSet';

function XChangeWindowAttributes (dpy: PDisplay; w: TWindow;
  ValueMask: MedCard; var Attrib: TXSetWindowAttributes): Integer;
asmname 'XChangeWindowAttributes';

type
  PPredicateFunc = ^function (dpy: PDisplay; Evt: PXEvent; Arg: TXPointer): TBool;

function XCheckIfEvent (dpy: PDisplay; var EvtRet: TXEvent;
  predicate: PPredicateFunc; Arg: TXPointer): TBool;
asmname 'XCheckIfEvent';

function XCheckMaskEvent (dpy: PDisplay; EvtMask: MedInt;
  var EvtRet: TXEvent): TBool;
asmname 'XCheckMaskEvent';

function XCheckTypedEvent (dpy: PDisplay; EvtType: Integer;
  var EvtRet: TXEvent): TBool;
asmname 'XCheckTypedEvent';

function XCheckTypedWindowEvent (dpy: PDisplay; w: TWindow; EvtType: Integer;
  var EvtRet: TXEvent): TBool;
asmname 'XCheckTypedWindowEvent';

function XCheckWindowEvent (dpy: PDisplay; w: TWindow; EvtMask: MedInt;
  var EvtRet: TXEvent): TBool;
asmname 'XCheckWindowEvent';

function XCirculateSubwindows (dpy: PDisplay; w: TWindow;
  Direction: Integer): Integer;
asmname 'XCirculateSubwindows';

function XCirculateSubwindowsDown (dpy: PDisplay; W: TWindow): Integer;
asmname 'XCirculateSubwindowsDown';

function XCirculateSubwindowsUp (dpy: PDisplay; W: TWindow): Integer;
asmname 'XCirculateSubwindowsUp';

function XClearArea (dpy: PDisplay; w: TWindow; x, y: Integer;
  Width, Height: Cardinal; Exposures: TBool): Integer;
asmname 'XClearArea';

function XClearWindow (dpy: PDisplay; w: TWindow): Integer;
asmname 'XClearWindow';

function XCloseDisplay (dpy: PDisplay): Integer;
asmname 'XCloseDisplay';

function XConfigureWindow (dpy: PDisplay; w: TWindow; ValueMask: Cardinal;
  var Values: TXWindowChanges): Integer;
asmname 'XConfigureWindow';

function XConnectionNumber (dpy: PDisplay): Integer;
asmname 'XConnectionNumber';

function XConvertSelection (dpy: PDisplay; Selection, Target, Propert: TAtom;
  Requestor: TWindow; time: TTime): Integer;
asmname 'XConvertSelection';

function XCopyArea (dpy: PDisplay; src, dest: TDrawable; gc: PGC;
  SrcX, SrcY: Integer; Width, Height: Cardinal; DestX, DestY: Integer): Integer;
asmname 'XCopyArea';

function XCopyGC (dpy: PDisplay; gc: PGC; ValueMask: MedCard; dest: PGC): Integer;
asmname 'XCopyGC';

function XCopyPlane (dpy: PDisplay; src, dest: TDrawable; gc: PGC;
  srcX, srcY: Integer; Width, Height: Cardinal; destX, destY: Integer;
  Plane: MedCard): Integer;
asmname 'XCopyPlane';

function XDefaultDepth (dpy: PDisplay; ScreenNum: Integer): Integer;
asmname 'XDefaultDepth';

function XDefaultDepthOfScreen (screen: PScreen): Integer;
asmname 'XDefaultDepthOfScreen';

function XDefaultScreen (dpy: PDisplay): Integer;
asmname 'XDefaultScreen';

function XDefineCursor (dpy: PDisplay; w: TWindow; cursor: TCursor): Integer;
asmname 'XDefineCursor';

function XDeleteProperty (dpy: PDisplay; w: TWindow; Propert: TAtom): Integer;
asmname 'XDeleteProperty';

function XDestroyWindow (dpy: PDisplay; w: TWindow): Integer;
asmname 'XDestroyWindow';

function XDestroySubwindows (dpy: PDisplay; w: TWindow): Integer;
asmname 'XDestroySubwindows';

function XDoesBackingStore (screen: PScreen): Integer;
asmname 'XDoesBackingStore';

function XDoesSaveUnders (screen: PScreen): TBool;
asmname 'XDoesSaveUnders';

function XDisableAccessControl (dpy: PDisplay): Integer;
asmname 'XDisableAccessControl';

function XDisplayCells (dpy: PDisplay; ScreenNum: Integer): Integer;
asmname 'XDisplayCells';

function XDisplayHeight (dpy: PDisplay; ScreenNum: Integer): Integer;
asmname 'XDisplayHeight';

function XDisplayHeightMM (dpy: PDisplay; ScreenNum: Integer): Integer;
asmname 'XDisplayHeightMM';

function XDisplayKeycodes (dpy: PDisplay; var MinKeycodesRet,
  MaxKeycodesRet: Integer): Integer;
asmname 'XDisplayKeycodes';

function XDisplayPlanes (dpy: PDisplay; ScreenNum: Integer): Integer;
asmname 'XDisplayPlanes';

function XDisplayWidth (dpy: PDisplay; ScreenNum: Integer): Integer;
asmname 'XDisplayWidth';

function XDisplayWidthMM (dpy: PDisplay; ScreenNum: Integer): Integer;
asmname 'XDisplayWidthMM';

function XDrawArc (dpy: PDisplay; d: TDrawable; gc: PGC; x, y: Integer;
  Width, Height: Cardinal; Angle1, Angle2: Integer): Integer;
asmname 'XDrawArc';

const
  XArcsArrayMax = High (Integer) div SizeOf (TXArc);
type
  PXArcsArray = ^TXArcsArray;
  TXArcsArray = array [0 .. XArcsArrayMax] of TXArc;

function XDrawArcs (dpy: PDisplay; d: TDrawable; gc: PGC;
  arcs: PXArcsArray; NumArcs: Integer): Integer;
asmname 'XDrawArcs';

function XDrawImageString (dpy: PDisplay; d: TDrawable; gc: PGC;
  x, y: Integer; aString: PChar; len: Integer): Integer;
asmname 'XDrawImageString';

function XDrawImageString16 (dpy: PDisplay; d: TDrawable; gc: PGC;
  x, y: Integer; aString: PXChar2b; len: Integer): Integer;
asmname 'XDrawImageString16';

function XDrawLine (dpy: PDisplay; d: TDrawable; gc: PGC;
  x0, y0, x1, y1: Integer): Integer;
asmname 'XDrawLine';

const
  XPointsArrayMax = High (Integer) div SizeOf (TXPoint);
type
  PXPointsArray = ^TXPointsArray;
  TXPointsArray = array [0 .. XPointsArrayMax] of TXPoint;

function XDrawLines (dpy: PDisplay; d: TDrawable; gc: PGC;
  points: PXPointsArray; NumPoints, Mode: Integer): Integer;
asmname 'XDrawLines';

function XDrawPoint (dpy: PDisplay; d: TDrawable; gc: PGC;
  x, y: Integer): Integer;
asmname 'XDrawPoint';

function XDrawPoints (dpy: PDisplay; d: TDrawable; gc: PGC;
  points: PXPointsArray; NumPoints, Mode: Integer): Integer;
asmname 'XDrawPoints';

function XDrawRectangle (dpy: PDisplay; d: TDrawable; gc: PGC;
  x, y: Integer; Width, Height: Cardinal): Integer;
asmname 'XDrawRectangle';

const
  XRectanglesArrayMax = High (Integer) div SizeOf (TXRectangle);
type
  PXRectanglesArray = ^TXRectanglesArray;
  TXRectanglesArray = array [0 .. XRectanglesArrayMax] of TXRectangle;

function XDrawRectangles (dpy: PDisplay; d: TDrawable; gc: PGC;
  rects: PXRectanglesArray; NumRects: Integer): Integer;
asmname 'XDrawRectangles';

const
  XSegmentsArrayMax = High (Integer) div SizeOf (TXSegment);
type
  PXSegmentsArray = ^TXSegmentsArray;
  TXSegmentsArray = array [0 .. XSegmentsArrayMax] of TXSegment;

function XDrawSegments (dpy: PDisplay; d: TDrawable; gc: PGC;
  segments: PXSegmentsArray; NumSegments: Integer): Integer;
asmname 'XDrawSegments';

function XDrawString (dpy: PDisplay; d: TDrawable; gc: PGC; x, y: Integer;
  s: PChar; len: Integer): Integer;
asmname 'XDrawString';

function XDrawString16 (dpy: PDisplay; d: TDrawable; gc: PGC; x, y: Integer;
  s: PXChar2b; len: Integer): Integer;
asmname 'XDrawString16';

const
  XTextItemsArrayMax = High (Integer) div SizeOf (TXTextItem);
type
  PXTextItemsArray = ^TXTextItemsArray;
  TXTextItemsArray = array [0 .. XTextItemsArrayMax] of TXTextItem;

function XDrawText (dpy: PDisplay; d: TDrawable; gc: PGC; x, y: Integer;
  s: PXTextItemsArray; NumItems: Integer): Integer;
asmname 'XDrawText';

const
  XTextItem16sArrayMax = High (Integer) div SizeOf (TXTextItem16);
type
  PXTextItem16sArray = ^TXTextItem16sArray;
  TXTextItem16sArray = array [0 .. XTextItem16sArrayMax] of TXTextItem16;

function XDrawText16 (dpy: PDisplay; d: TDrawable; gc: PGC; x, y: Integer;
  s: PXTextItem16sArray; NumItems: Integer): Integer;
asmname 'XDrawText16';

function XEnableAccessControl (dpy: PDisplay): Integer;
asmname 'XEnableAccessControl';

function XEventsQueued (dpy: PDisplay; Mode: Integer): Integer;
asmname 'XEventsQueued';

function XFetchName (dpy: PDisplay; w: TWindow;
  var WinNameRet: PChar): Integer;
asmname 'XFetchName';

function XFillArc (dpy: PDisplay; d: TDrawable; gc: PGC; x, y: Integer;
  Width, Height: Cardinal; Angle1, Angle2: Integer): Integer;
asmname 'XFillArc';

function XFillArcs (dpy: PDisplay; d: TDrawable; gc: PGC;
  arcs: PXArcsArray; NumArcs: Integer): Integer;
asmname 'XFillArcs';

function XFillPolygon (dpy: PDisplay; d: TDrawable; gc: PGC;
  points: PXPointsArray; NPoints, Shape, Mode: Integer): Integer;
asmname 'XFillPolygon';

function XFillRectangle (dpy: PDisplay; d: TDrawable; gc: PGC;
  x, y: Integer; Width, Height: Cardinal): Integer;
asmname 'XFillRectangle';

function XFillRectangles (dpy: PDisplay; d: TDrawable; gc: PGC;
  rectangles: PXRectanglesArray; NumRets: Integer): Integer;
asmname 'XFillRectangles';

function XFlush (dpy: PDisplay): Integer;
asmname 'XFlush';

function XForceScreenSaver (dpy: PDisplay; Mode: Integer): Integer;
asmname 'XForceScreenSaver';

function XFree (Data: Pointer): Integer;
asmname 'XFree';

function XFreeColormap (dpy: PDisplay; colormap: TColormap): Integer;
asmname 'XFreeColormap';

const
  MedCardsArrayMax = High (Integer) div SizeOf (MedCard);
type
  PMedCardsArray = ^TMedCardsArray;
  TMedCardsArray = array [0 .. MedCardsArrayMax] of MedCard;

function XFreeColors (dpy: PDisplay; colormap: TColormap; 
  Pixels: PMedCardsArray; NumPixel: Integer; Planes: MedCard): Integer;
asmname 'XFreeColors';

function XFreeCursor (dpy: PDisplay; cursor: TCursor): Integer;
asmname 'XFreeCursor';

{ @@ I'm not sure with the type, Could be of type PCharsetListArray }
function XFreeExtensionList (var list: PChar): Integer;
asmname 'XFreeExtensionList';

function XFreeFont (dpy: PDisplay; fontstruct: PXFontStruct): Integer;
asmname 'XFreeFont';

function XFreeFontInfo (names: PCharsetListArray; freeinfo: PXFontStruct;
  actualCount: Integer): Integer;
asmname 'XFontStruct';

function XFreeFontNames (list: PCharsetListArray): Integer;
asmname 'XFreeFontNames';

function XFreeFontPath (list: PCharsetListArray): Integer;
asmname 'XFreeFontPath';

function XFreeGC (dpy: PDisplay; gc: PGC): Integer;
asmname 'XFreeGC';

function XFreeModifiermap (modmap: PXModifierKeymap): Integer;
asmname 'XFreeModifiermap';

function XFreePixmap (dpy: PDisplay; pixmap: TPixmap): Integer;
asmname 'XFreePixmap';

function XGeometry (dpy: PDisplay; screennum: Integer; pos, defaultpos: PChar;
  bWidth, fWidth, fHeight: Cardinal; xAdder, yAdder: Integer;
  var xRet, yRet, WidthRet, HeightRet: Integer): Integer;
asmname 'XGeometry';

function XGetErrorDatabaseText (dpy: PDisplay; name, Msg, DefStr: PChar;
  BufferRet: PChar; len: Integer): Integer;
asmname 'XGetErrorDatabaseText';

function XGetErrorText (dpy: PDisplay; ErrCode: Integer; BuffRet: PChar;
  Len: Integer): Integer;
asmname 'XGetErrorText';

function XGetFontProperty (var fontStruct: TXFontStruct; atom: TAtom;
  var ValueRet: MedCard): TBool;
asmname 'XGetFontProperty';

function XGetGCValues (dpy: PDisplay; gc: PGC; valuemask: MedCard;
  var ValRet: PXGCValues): TStatus;
asmname 'XGetGCValues';

function XGetGeometry (dpy: PDisplay; d: TDrawable; var RootWindow: TWindow;
  var x, y: Integer; var Width, Height, BorderWidth, Depth: Cardinal): TStatus;
asmname 'XGetGeometry';

function XGetIconName (dpy: PDisplay; w: TWindow; var Name: PChar): TStatus;
asmname 'XGetIconName';

function XGetInputFocus (dpy: PDisplay; var Focus: TWindow; var Revert:
  Integer): Integer;
asmname 'XGetInputFocus';

function XGetKeyboardControl (dpy: PDisplay; var Values: TXKeyboardState): Integer;
asmname 'XGetKeyboardControl';

function XGetPointerControl (dpy: PDisplay; var Nominator, Denominitaor,
  Treshold: Integer): Integer;
asmname 'XGetPointerControl';

const
  ByteCardsArrayMax = High (Integer) div SizeOf (ByteCard);
type
  PByteCardsArray = ^TByteCardsArray;
  TByteCardsArray = array [0 .. ByteCardsArrayMax] of ByteCard;

function XGetPointerMapping (dpy: PDisplay; MapRet: PByteCardsArray;
  NMap: Integer): Integer;
asmname 'XGetPointerMapping';

function XGetScreenSaver (dpy: PDisplay; var TimeoutRet, IntervalRet,
  PreferBlankingRet, AllowExposRet: Integer): Integer;
asmname 'XGetScreenSaver';

function XGetTransientForHint (dpy: PDisplay; w: TWindow;
  var PropWinRet: TWindow): TStatus;
asmname 'XGetTransientForHint';

{ @@ Not sure with PropRet}
function XGetWindowProperty (dpy: PDisplay; w: TWindow; Propert: TAtom;
  LongOffset, LongLength: MedInt; Del: TBool; ReqType: TAtom;
  var ActualTypeRet: TAtom; var ActualFormatRet: Integer;
  var NItemsRet, BytesAfterRet: MedCard; var PropRet: PChar): Integer;
asmname 'XGetWindowProperty';

function XGetWindowAttributes (dpy: PDisplay; w: TWindow;
  var WinAttrRet: TXWindowAttributes): TSTatus;
asmname 'XGetWindowAttributes';

function XGrabButton (dpy: PDisplay; Button, Modifier: Cardinal;
  GrabWin: TWindow; OwnEvts: TBool; EvtMask: Cardinal;
  PointerMode, KeyboardMode: Integer; ConfineTo: TWindow;
  Cursor: TCursor): Integer;
asmname 'XGrabButton';

function XGrabKey (dpy: PDisplay; keycode: Integer; Modifiers: Cardinal;
  GrabWin: TWindow; OwnerEvt: TBool; PointerMode, KeyboardMode: Integer): Integer;
asmname 'XGrabKey';

function XGrabKeyboard (dpy: PDisplay; GrabWin: TWindow; OwnEvts: TBool;
  PointerMode, KeyboardMode: Integer; time: TTIme): Integer;
asmname 'XGrabKeyboard';

function XGrabPointer (dpy: PDisplay; GrabWin: TWindow; OwnEvts: TBool;
  EventMask: Cardinal; PointerMode, KeyboardMode: Integer; ConfineTo: TWindow;
  cursor: TCursor; time: TTime): Integer;
asmname 'XGrabPointer';

function XGrabServer (dpy: PDisplay): Integer;
asmname 'XGrabServer';

function XHeightMMOfScreen (screen: PScreen): Integer;
asmname 'XHeightMMOfScreen';

function XHeightOfScreen (screen: PScreen): Integer;
asmname 'XHeightOfScreen';

function XIfEvent (dpy: PDisplay; var EvtRet: TXEvent; predicate: PPredicateFunc;
  Arg: TXPointer): Integer;
asmname 'XIfEvent';

function XImageByteOrder (dpy: PDisplay): Integer;
asmname 'XImageByteOrder';

function XInstallColormap (dpy: PDisplay; ColMap: TColormap): Integer;
asmname 'XInstallColormap';

function XKeysymToKeycode (dpy: PDisplay; keysym: TKeysym): TKeyCode;
asmname 'XKeysymToKeycode';

function XKillClient (dpy: PDisplay; Resource: TXID): Integer;
asmname 'XKillClient';

function XLookupColor (dpy: PDisplay; Colmap: TColormap; ColorName: PChar;
  var ExactDefRet, ScreenDefRet: TXColor): TStatus;
asmname 'XLookupColor';

function XLowerWindow (dpy: PDisplay; w: TWindow): Integer;
asmname 'XLowerWindow';

function XMapRaised (dpy: PDisplay; w: TWindow): Integer;
asmname 'XMapRaised';

function XMapSubwindows (dpy: PDisplay; w: TWindow): Integer;
asmname 'XMapSubwindows';

function XMapWindow (dpy: PDisplay; w: TWindow): Integer;
asmname 'XMapWindow';

function XMaskEvent (dpy: PDisplay; EvtMask: MedInt; var EvtRet: TXEvent): Integer;
asmname 'XMaskEvent';

function XMaxCmapsOfScreen (screen: PScreen): Integer;
asmname 'XMaxCmapsOfScreen';

function XMinCmapsOfScreen (screen: PSCreen): Integer;
asmname 'XMinCmapsOfScreen';

function XMoveResizeWindow (dpy: PDisplay; w: TWindow; x, y: Integer;
  Width, Height: Cardinal): Integer;
asmname 'XMoveResizeWindow';

function XMoveWindow (dpy: PDisplay; w: TWindow; x, y: Integer): Integer;
asmname 'XMoveWindow';

function XNextEvent (dpy: PDisplay; var EvtRet: TXEvent): Integer;
asmname 'XNextEvent';

function XNoOp (dpy: PDisplay): Integer;
asmname 'XNoOp';

function XParseColor (dpy: PDisplay; colmap: TColormap; spec: PChar;
  var ExactDefRet: TXColor): TSTatus;
asmname 'XParseColor';

function XParseGeometry (ParseStr: PChar; var x, y: Integer;
  var Width, Height: Cardinal): Integer;
asmname 'XParseGeometry';

function XPeekEvent (dpy: PDisplay; var evtRet: TXEvent): Integer;
asmname 'XPeekEvent';

function XPeekIfEvent (dpy: PDisplay; var EvtRet: TXEvent;
  Predicate: PPredicateFunc; Arg: TXPointer): Integer;
asmname 'XPeekIfEvent';

function XPending (dpy: PDisplay): Integer;
asmname 'XPending';

function XPlanesOfScreen (screen: PScreen): Integer;
asmname 'XPlanesOfScreen';

function XProtocolRevision (dpy: PDisplay): Integer;
asmname 'XProtocolRevision';

function XProtocolVersion (dpy: PDisplay): Integer;
asmname 'XProtocolVersion';

function XPutBackEvent (dpy: PDisplay; Evt: PXEvent): Integer;
asmname 'XPutBackEvent';

function XPutImage (dpy: PDisplay; d: TDrawable; gc: PGC; image: PXImage;
  srcX, srcY, destX, destY: Integer; Width, Height: Cardinal): Integer;
asmname 'XPutImage';

function XQLength (dpy: PDisplay): Integer;
asmname 'XQLength';

function XQueryBestCursor (dpy: PDisplay; d: TDrawable; Width, Height: Cardinal;
  var WidthRet, HeightRet: Cardinal): TStatus;
asmname 'XQueryBestCursor';

function XQueryBestSize (dpy: PDisplay; Klass: Integer; WhichScreen: TDrawable;
  Width, Height: Cardinal; var WithRet, HeightRet: Cardinal): TStatus;
asmname 'XQueryBestSize';

function XQueryBestStipple (dpy: PDisplay; WhichScreen: TDrawable;
  Width, Height: Cardinal; var WithRet, HeightRet: Cardinal): TStatus;
asmname 'XQueryBestStipple';

function XQueryBestTile (dpy: PDisplay; WhichScreen: TDrawable;
  Width, Height: Cardinal; var WithRet, HeightRet: Cardinal): TStatus;
asmname 'XQueryBestTile';

function XQueryColor (dpy: PDisplay; colmap: TColormap; var color: TXColor): Integer;
asmname 'XQueryColor';

const
  XColorsArrayMax = High (Integer) div SizeOf (TXColor);
type
  PXColorsArray = ^TXColorsArray;
  TXColorsArray = array [0 .. XColorsArrayMax] of TXColor;

function XQueryColors (dpy: PDisplay; colormap: TColormap; 
  Colors: PXColorsArray; NumColors: Integer): Integer;
asmname 'XQueryColors';

function XQueryExtension (dpy: PDisplay; Name: PChar; var MajorOpcodeRet,
  FirstEvtRet, FirstErrRet: Integer): TBool;
asmname 'XQueryExtension';

type
  TChar32Array = array [0 .. 31] of Char;

function XQueryKeymap (dpy: PDisplay; KeysRet: TChar32Array): Integer;
asmname 'XQueryKeymap';

function XQueryPointer (dpy: PDisplay; w: TWindow; var RootRet, ChildRet: TWindow;
  var RootXRet, RootYRet, WinXRet, WinYRet: Integer; var MaskRet: Cardinal): TBool;
asmname 'XQueryPointer';

function XQueryTextExtents (dpy: PDisplay; FontID: TXID; aString: PChar;
  NChars: Integer; var DirRet, FontAscendRet, FontDescentRet: Integer;
  var OverallRet: TXCharStruct): Integer;
asmname 'XQueryTextExtents';

function XQueryTextExtents16 (dpy: PDisplay; FontID: TXID; aString: PXChar2b;
  NChars: Integer; var DirRet, FontAscendRet, FontDescentRet: Integer;
  var OverallRet: TXCharStruct): Integer;
asmname 'XQueryTextExtents';

function XQueryTree (dpy: PDisplay; w: TWindow; var RootRet, ParentRet: TWindow;
  var ChildrenRet: PWindowsArray; var NChildRet: Cardinal): TStatus;
asmname 'XQueryTree';

function XRaiseWindow (dpy: PDisplay; w: TWindow): Integer;
asmname 'XRaiseWindow';

function XReadBitmapFile (dpy: PDisplay; d: TDrawable; FileName: PChar;
  var WidthRet, HeightRet: Cardinal; var BitmapRet: TPixmap; var XHotRet,
  YHotRet: Integer): Integer;
asmname 'XReadBitmapFile';

function XReadBitmapFileData (FileName: PChar; var WidthRet, HeightRet: Cardinal;
  var DataRet: PByteCardsArray; var XHotRet, YHotRet: Integer): Integer;
asmname 'XReadBitmapFileData';

type
  PByteCard = ^ByteCard;

function XRebindKeysym (dpy: PDisplay; keysym: TKeysym; aList: PKeysymsArray;
  ModCount: Integer; aString: PByteCard; BytesString: Integer): Integer;
asmname 'XRebindKeysym';

function XRecolorCursor (dpy: PDisplay; cursor: TCursor;
  FgColor, BgColor: PXColor): Integer;
asmname 'XRecolorCursor';

function XRefreshKeyboardMapping (var EvtMap: TXMappingEvent): Integer;
asmname 'XRefreshKeyboardMapping';

function XRemoveFromSaveSet (dpy: PDisplay; w: TWindow): Integer;
asmname 'XRemoveFromSaveSet';

function XRemoveHost (dpy: PDisplay; Host: PXHostAddress): Integer;
asmname 'XRemoveHost';

function XRemoveHosts (dpy: PDisplay; Hosts: PXHostAddressArray;
  NumHosts: Integer): Integer;
asmname 'XRemoveHosts';

function XReparentWindow (dpy: PDisplay; w, Parent: TWindow; x, y: Integer): Integer;
asmname 'XReparentWindow';

function XResetScreenSaver (dpy: PDisplay): Integer;
asmname 'XResetScreenSaver';

function XResizeWindow (dpy: PDisplay; w: TWindow; Width, Height: Integer): Integer;
asmname 'XResizeWindow';

function XRestackWindows (dpy: PDisplay; Windows: PWindowsArray;
  NumWin: Integer): Integer;
asmname 'XRestackWindows';

function XRotateBuffers (dpy: PDisplay; Rotate: Integer): Integer;
asmname 'XRotateBuffers';

function XRotateWindowProperties (dpy: PDisplay; w: TWindow; Properties:
  PAtomsArray; NumProp, NPositions: Integer): Integer;
asmname 'XRotateWindowProperties';

function XScreenCount (dpy: PDisplay): Integer;
asmname 'XScreenCount';

function XSelectInput (dpy: PDisplay; w: TWindow; EvtMask: MedInt): Integer;
asmname 'XSelectInput';

function XSendEvent (dpy: PDisplay; w: TWindow; propagate: TBool;
  EvtMask: MedInt; EvtSend: PXEvent): TStatus;
asmname 'XSendEvent';

function XSetAccessControl (dpy: PDisplay; Mode: Integer): Integer;
asmname 'XSetAccessControl';

{ ArcMode can have ArcChord or ArcPieSlice }
function XSetArcMode (dpy: PDisplay; gc: PGC; ArcMode: Integer): Integer;
asmname 'XSetArcMode';

function XSetBackground (dpy: PDisplay; gc: PGC; Background: MedCard): Integer;
asmname 'XSetBackground';

function XSetClipMask (dpy: PDisplay; gc: PGC; pixmap: TPixmap): Integer;
asmname 'XSetClipMask';

function XSetClipOrigin (dpy: PDisplay; gc: PGC; ClipXOrigin,
  ClipYOrigin: Integer): Integer;
asmname 'XSetClipOrigin';

function XSetClipRectangles (dpy: PDisplay; gc: PGC; ClipXOrigin,
  ClipYOrigin: Integer; rectangles: PXRectanglesArray; N,
  Ordering: Integer): Integer;
asmname 'XRectangle';

function XSetCloseDownMode (dpy: PDisplay; CloseMode: Integer): Integer;
asmname 'XSetCloseDownMode';

function XSetCommand (dpy: PDisplay; w: TWindow; aArgv: PCharsetListArray;
  aArgc: Integer): Integer;
asmname 'XSetCommand';

function XSetDashes (dpy: PDisplay; gc: PGC; DashOffset: Integer;
  DashList: PChar; Num: Integer): Integer;
asmname 'XSetDashes';

function XSetFillRule (dpy: PDisplay; gc: PGC; FillRule: Integer): Integer;
asmname 'XSetFillRule';

function XSetFillStyle (dpy: PDisplay; gc: PGC; FillStyle: Integer): Integer;
asmname 'XSetFillStyle';

function XSetFont (dpy: PDisplay; gc: PGC; font: TFont): Integer;
asmname 'XSetFont';

function XSetFontPath (dpy: PDisplay; Directories: PCharsetListArray;
  NumDirs: Integer): Integer;
asmname 'XSetFontPath';

function XSetForeground (dpy: PDisplay; gc: PGC; Foreground: MedCard): Integer;
asmname 'XSetForeground';

function XSetFunction (dpy: PDisplay; gc: PGC; aFunction: Integer): Integer;
asmname 'XSetFunction';

function XSetGraphicsExposures (dpy: PDisplay; gc: PGC; GraphExp: TBool): Integer;
asmname 'XSetGraphicsExposures';

function XSetIconName (dpy: PDisplay; w: TWindow; IconName: PChar): Integer;
asmname 'XSetIconName';

function XSetInputFocus (dpy: PDisplay; Focus: TWindow; RevertTo: Integer;
  Time: TTime): Integer;
asmname 'XSetInputFocus';

function XSetLineAttributes (dpy: PDisplay; gc: PGC; LineWidth: Cardinal;
  LineStyle, CapStyle, JoinStyle: Integer): Integer;
asmname 'XSetLineAttributes';

function XSetModifierMapping (dpy: PDisplay; ModMap: PXModifierKeymap): Integer;
asmname 'XSetModifierMapping';

function XSetPlaneMask (dpy: PDisplay; gc: PGC; PlaneMask: MedCard): Integer;
asmname 'XSetPlaneMask';

function XSetPointerMapping (dpy: PDisplay; Map: PByteCard;
  NMap: Integer): Integer;
asmname 'XSetPointerMapping';

function XSetScreenSaver (dpy: PDisplay; Timeout, Interval, PrefBlank,
  AllowExp: Integer): Integer;
asmname 'XSetScreenSaver';

function XSetSelectionOwner (dpy: PDisplay; Selection: TAtom; Owner: TWindow;
  Time: TTime): Integer;
asmname 'XSetSelectionOwner';

function XSetState (dpy: PDisplay; gc: PGC; Foreground, Background: MedCard;
  functio: Integer; PlaneMask: MedCard): Integer;
asmname 'XSetState';

function XSetStipple (dpy: PDisplay; gc: PGC; Stripple: TPixmap): Integer;
asmname 'XSetStipple';

function XSetSubwindowMode (dpy: PDisplay; gc: PGC;
  SubWindowMode: Integer): Integer;
asmname 'XSetSubwindowMode';

function XSetTSOrigin (dpy: PDisplay; gc: PGC; TsXOrigin,
  TsYOrigin: Integer): Integer;
asmname 'XSetTSOrigin';

function XSetTile (dpy: PDisplay; gc: PGC; Tile: TPixmap): Integer;
asmname 'XSetTile';

function XSetWindowBackground (dpy: PDisplay; Window: TWindow;
  BackgroundPixel: MedCard): Integer;
asmname 'XSetWindowBackground';

function XSetWindowBackgroundPixmap (dpy: PDisplay; Window: TWindow;
  BackgroundPixmap: TPixmap): Integer;
asmname 'XSetWindowBackgroundPixmap';

function XSetWindowBorder (dpy: PDisplay; Window: TWindow; 
  BorderPixel: MedCard): Integer;
asmname 'XSetWindowBorder';

function XSetWindowBorderPixmap (dpy: PDisplay; Window: TWindow;
  BorderPixmap: TPixmap): Integer;
asmname 'XSetWindowBorderPixmap';

function XSetWindowBorderWidth (dpy: PDisplay; Window: TWindow;
  Width: Cardinal): Integer;
asmname 'XSetWindowBorderWidth';

function XSetWindowColormap (dpy: PDisplay; Window: TWindow;
  Colormap: TColormap): Integer;
asmname 'XSetWindowColormap';

function XStoreBuffer (dpy: PDisplay; Bytes: PChar; NBytes,
  Buffer: Integer): Integer;
asmname 'XStoreBuffer';

function XStoreBytes (dpy: PDisplay; Bytes: PChar;
  NBytes: Integer): Integer;
asmname 'XStoreBytes';

function XStoreColor (dpy: PDisplay; ColorMap: TColormap;
  Color: PXColor): Integer;
asmname 'XStoreColor';

function XStoreColors (dpy: PDisplay; ColorMap: TColorMap;
  Color: PXColor; NColors: Integer): Integer;
asmname 'XStoreColors';

function XStoreName (dpy: PDisplay; Window: TWindow;
  WindowName: PChar): Integer;
asmname 'XStoreName';

function XStoreNamedColor (dpy: PDisplay; ColorMap: TColorMap;
  Color: PChar; Pixel: MedCard; Flags: Integer): Integer;
asmname 'XStoreNamedColor';

function XSync (dpy: PDisplay; Discard: TBool): Integer;
asmname 'XSync';

function XTextExtents (FontStruct: PXFontStruct; aString: PChar;
  NChars: Integer; var DirectionRet, FontAscentRet,
  FontDescentRet: Integer; var OverallRet: TXCharStruct): Integer;
asmname 'XTextExtents';

function XTextExtents16 (FontStruct: PXFontStruct; aString16: PXChar2B;
  NChars: Integer; var DirectionRet, FontAscentRet,
  FontDescentRet: Integer; var OverallRet: TXCharStruct): Integer;
asmname 'XTextExtents16';

function XTextWidth (FontStruct: PXFontStruct; aString: PChar;
  Count: Integer): Integer;
asmname 'XTextWidth';

function XTextWidth16 (FontStruct: PXFontStruct; aString16: PXChar2B;
  Count: Integer): Integer;
asmname 'XTextWidth16';

function XTranslateCoordinates (dpy: PDisplay; SrcWin, DestWin: TWindow;
  SrcX, SrcY: Integer; var DestXRet, DestYRet: Integer;
  var ChildRet: TWindow): TBool;
asmname 'XTranslateCoordinates';

function XUndefineCursor (dpy: PDisplay; Window: TWindow): Integer;
asmname 'XUndefineCursor';

function XUngrabButton (dpy: PDisplay; Button, Modifiers: Cardinal;
  GrabWindow: TWindow): Integer;
asmname 'XUngrabButton';

function XUngrabKey (dpy: PDisplay; KeyCode: Integer; Modifiers: Cardinal;
  GrabWindow: TWindow): Integer;
asmname 'XUngrabKey';

function XUngrabKeyboard (dpy: PDisplay; Time: TTime): Integer;
asmname 'XUngrabKeyboard';

function XUngrabPointer (dpy: PDisplay; Time: TTime): Integer;
asmname 'XUngrabPointer';

function XUngrabServer (dpy: PDisplay): Integer;
asmname 'XUngrabServer';

function XUninstallColormap (dpy: PDisplay; Colormap: TColorMap): Integer;
asmname 'XUninstallColormap';

function XUnloadFont (dpy: PDisplay; Font: TFont): Integer;
asmname 'XUnloadFont';

function XUnmapSubwindows (dpy: PDisplay; Window: TWindow): Integer;
asmname 'XUnmapSubwindows';

function XUnmapWindow (dpy: PDisplay; Window: TWindow): Integer;
asmname 'XUnmapWindow';

function XVendorRelease (dpy: PDisplay): Integer;
asmname 'XVendorRelease';

function XWarpPointer (dpy: PDisplay; SrcWin, DestWin: TWindow;
  SrcX, SrcY: Integer; SrcWidth, SrcHEight: Cardinal;
  DestX, DestY: Integer): Integer;
asmname 'XWarpPointer';

function XWidthMMOfScreen (Screen: PScreen): Integer;
asmname 'XWidthMMOfScreen';

function XWidthOfScreen (Screen: PScreen): Integer;
asmname 'XWidthOfScreen';

function XWindowEvent (dpy: PDisplay; Window: TWindow;
  EventMask: MedInt; var EvtRet: TXEvent): Integer;
asmname 'XWindowEvent';

function XWriteBitmapFile (dpy: PDisplay; Path: PChar; Bitmap: TPixmap;
  Width, Height: Cardinal; XHot, YHot: Integer): Integer;
asmname 'XWriteBitmapFile';

function XSupportsLocale: TBool;
asmname 'XSupportsLocale';

function XSetLocaleModifiers (ModifierList: PChar): PChar;
asmname 'XSetLocaleModifiers';

{ @@ rdb is struct _XrmHashBucketRec* }
function XOpenOM (dpy: PDisplay; rdb: Pointer;
  ResName, ResClass: PChar): PXOM;
asmname 'XOpenOM';

function XCloseOM (om: PXOM): TStatus;
asmname 'XCloseOM';

function XSetOMValues (om: PXOM; ...): PChar;
asmname 'XSetOMValues';

function XGetOMValues (om: PXOM; ...): PCHar;
asmname 'XGetOMValues';

function XDisplayOfOM (om: PXOM): PDisplay;
asmname 'XDisplayOfOM';

function XLocaleOfOM (om: PXOM): PChar;
asmname 'XLocaleOfOM';

function XCreateOC (om: PXOM; ...): PXOC;
asmname 'XCreateOC';

procedure XDestroyOC (oc: PXOC);
asmname 'XDestroyOC';

function XOMOfOC (oc: PXOC): PXOM;
asmname 'XOMOfOC';

function XSetOCValues (oc: PXOC; ...): PChar;
asmname 'XSetOCValues';

function XGetOCValues (oc: PXOC; ...): PChar;
asmname 'XGetOCValues';

function XCreateFontSet (dpy: PDisplay; BaseFontNameList: PChar;
  var MissingCharsetList: PCharsetListArray;
  var MissingCharsetCount: Integer; 
  DefString: PCharsetListArray): PXFontSet;
asmname 'XCreateFontSet';

procedure XFreeFontSet (dpy: PDisplay; FontSet: PXFontSet);
asmname 'XFreeFontSet';

function XFontsOfFontSet (FontSet: PXFontSet;
  var FontStructList: PPXFontStruct; 
  var FontNameList: PCHarsetListArray): Integer;
asmname 'XFontsOfFontSet';

function XBaseFontNameListOfFontSet (FontSet: PXFontSet): PChar;
asmname 'XBaseFontNameListOfFontSet';

function XLocaleOfFontSet (FontSet: PXFontSet): PChar;
asmname 'XLocaleOfFontSet';

function XContextDependentDrawing (FontSet: PXFontSet): TBool;
asmname 'XContextDependentDrawing';

function XDirectionalDependentDrawing (FontSet: PXFontSet): TBool;
asmname 'XDirectionalDependentDrawing';

function XContextualDrawing (FontSet: PXFontSet): TBool;
asmname 'XContextualDrawing';

function XExtentsOfFontSet (FontSet: PXFontSet): PXFontSetExtents;
asmname 'XExtentsOfFontSet';

function XmbTextEscapement (FontSet: PXFontSet; Text: PChar;
  BytesText: Integer): Integer;
asmname 'XmbTextEscapement';

function XwcTextEscapement (FontSet: PXFontSet; aText: PWCharT;
  NumWChars: Integer): Integer;
asmname 'XwcTextEscapement';

function XmbTextExtents (FontSet: PXFontSet; aText: PChar;
  BytesText: Integer;
  var OverallInkReturn, OverallLogicalReturn: TXRectangle): Integer;
asmname 'XmbTextExtents';

function XwcTextExtents (FontSet: PXFontSet; aText: PWCharT;
  NumWChars: Integer; 
  var OverallInkReturn, OverallLogicalReturn: TXRectangle): Integer;
asmname 'XwcTextExtents';

function XmbTextPerCharExtents (FontSet: PXFontSet; aText: PChar;
  BytesText: Integer; 
  InkExtendsBuffer, LogicalExtendsBuffer: PXRectangle;
  BufferSize: Integer; var NumChars: Integer; 
  var OverallInkReturn, OverallLogicalReturn: TXRectangle): TStatus;
asmname 'XmbTextPerCharExtents';

function XwcTextPerCharExtents (FontSet: PXFontSet; aText: PWCharT;
  NumWChars: Integer; 
  InkExtendsBuffer, LogicalExtendsBuffer: PXRectangle;
  BufferSize: Integer; var NumChars: Integer; 
  var OverallInkReturn, OverallLogicalReturn: TXRectangle): TStatus;
asmname 'XwcTextPerCharExtents';

procedure XmbDrawText (dpy: PDisplay; d: TDrawable; gc: PGC;
  x, y: Integer; TextItems: PXmbTextItem; NItems: Integer);
asmname 'XmbDrawText';

procedure XwcDrawText (dpy: PDisplay; d: TDrawable; gc: PGC;
  x, y: Integer; TextItems: PXWcTExtItem);
asmname 'XwcDrawText';

procedure XmbDrawString (dpy: PDisplay; d: TDrawable; FontSet: PXFontSet;
  gc: PGC; x, y: Integer; aText: PChar; BytesText: Integer);
asmname 'XmbDrawString';

procedure XwcDrawString (dpy: PDisplay; d: TDrawable; FontSet: PXFontSet;
  gc: PGC; x, y: Integer; aText: PWCharT; NumWChars: Integer);
asmname 'XwcDrawString';

procedure XmbDrawImageString (dpy: PDisplay; d: TDrawable;
  FontSet: PXFontSet; gc: PGC; x, y: Integer; aText: PChar;
  BytesText: Integer);
asmname 'XmbDrawImageString';

procedure XwcDrawImageString (dpy: PDisplay; d: TDrawable;
  FontSet: PXFontSet; gc: PGC; x, y: Integer; aText: PWCharT;
  NumWChars: Integer);
asmname 'XwcDrawImageString';

{ @@ rdb is struct _XrmHashBucketRec* }
function XOpenIM (dpy: PDisplay; rdb: Pointer;
  ResName, ResClass: PChar): PXIM;
asmname 'XOpenIM';

function XCloseIM (im: PXIM): TStatus;
asmname 'XCloseIM';

function XGetIMValues (im: PXIM; ...): PChar;
asmname 'XGetIMValues';

function XSetIMValues (im: PXIM; ...): PChar;
asmname 'XSetIMValues';

function XDisplayOfIM (im: PXIM): PDisplay;
asmname 'XDisplayOfIM';

function XLocaleOfIM (im: PXIM): PChar;
asmname 'XLocaleOfIM';

function XCreateIC (im: PXIM; ...): PXIC;
asmname 'XCreateIC';

procedure XDestroyIC (ic: PXIC);
asmname 'XDestroyIC';

procedure XSetICFocus (ic: PXIC);
asmname 'XSetICFocus';

procedure XUnsetICFocus (ic: PXIC);
asmname 'XUnsetICFocus';

function XwcResetIC (ic: PXIC): PWCharT;
asmname 'XwcResetIC';

function XmbResetIC (ic: PXIC): PChar;
asmname 'XmbResetIC';

function XSetICValues (ic: PXIC; ...): PChar;
asmname 'XSetICValues';

function XGetICValues (ic: PXIC; ...): PChar;
asmname 'XGetICValues';

function XIMOfIC (ic: PXIC): PXIM;
asmname 'XIMOfIC';

function XFilterEvent (Event: PXEvent; Window: TWindow): TBool;
asmname 'XFilterEvent';

function XmbLookupString (ic: PXIC; Event: PXKEyPressedEvent;
  BufferRet: PChar; BytesBuffer: Integer; var KeysymRet: TKeySym;
  var StatusRet: TStatus): Integer;
asmname 'XmbLookupString';

function XwcLookupString (ic: PXIC; Event: PXKEyPressedEvent;
  BufferRet: PWCharT; WCHarsBuf: Integer; var KeysymRet: TKeySym;
  var StatusRet: TStatus): Integer;
asmname 'XwcLookupString';

{@subsection internal connections for IMs }

{ @@ rdb is struct _XrmHashBucketRec* }
function XRegisterIMInstantiateCallback (dpy: PDisplay; rdb: Pointer;
  ResName, ResClass: PChar; Callback: PXIDProc;
  ClientData: TXPointer): TBool;
asmname 'XRegisterIMInstantiateCallback';

{ @@ rdb is struct _XrmHashBucketRec* }
function XUnregisterIMInstantiateCallback (dpy: PDisplay; rdb: Pointer;
  ResName, ResClass: PChar; Callback: PXIDProc;
  ClientData: TXPointer): TBool;
asmname 'XUnregisterIMInstantiateCallback';

type
  PXConnectionWatchProc = ^procedure (dpy: PDisplay; ClientData: TXPointer;
    fd: Integer; Opening: TBool; WatchData: PXPointer);

function XInternalConnectionNumbers (dpy: PDisplay; var FdRet: TIntegerArray;
  var CountRet: Integer): TStatus;
asmname 'XInternalConnectionNumbers'; 

procedure XProcessInternalConnection (dpy: PDisplay; Fd: Integer);
asmname 'XProcessInternalConnection';

function XAddConnectionWatch (dpy: PDisplay;
  Callback: PXConnectionWatchProc; ClientData: TXPointer): TStatus;
asmname 'XAddConnectionWatch';

procedure XRemoveConnectionWatch (dpy: PDisplay;
  Callback: PXConnectionWatchProc; ClientData: TXPointer);
asmname 'XRemoveConnectionWatch';


implementation

{$L xlibc.c}
{$L X11}

{ functions for checking the type sizes }

function CSizeofWCharT: Cardinal;
asmname '__sizeof_wchar_t';
function PSizeofTWCharT: Cardinal;
begin
  PSizeofTWCharT := sizeof (TWCharT)
end;

function CSizeofXExtData: Cardinal;
asmname '__sizeof_xextdata';
function PSizeofTXExtData: Cardinal;
begin
  PSizeofTXExtData := sizeof (TXExtData)
end;

function CSizeofXExtCodes: Cardinal;
asmname '__sizeof_xextcodes';
function PSizeofTXExtCodes: Cardinal;
begin
  PSizeofTXExtCodes := sizeof (TXExtCodes)
end;

function CSizeofXPixmapFormatValues: Cardinal;
asmname '__sizeof_xpixmapformatvalues';
function PSizeofTXPixmapformatValues: Cardinal;
begin
  PSizeofTXPixmapFormatValues := sizeof (TXPixmapformatValues)
end;

function CSizeofXGcValues: Cardinal;
asmname '__sizeof_xgcvalues';
function PSizeofTXGcValues: Cardinal;
begin
  PSizeofTXGcValues := sizeof (TXGcValues)
end;

function CSizeofVisual: Cardinal;
asmname '__sizeof_visual';
function PSizeofTVisual: Cardinal;
begin
  PSizeofTVisual := sizeof (TVisual)
end;

function CSizeofDepth: Cardinal;
asmname '__sizeof_depth';
function PSizeofTDepth: Cardinal;
begin
  PSizeofTDepth := sizeof (TDepth)
end;

function CSizeofScreen: Cardinal;
asmname '__sizeof_screen';
function PSizeofTScreen: Cardinal;
begin
  PSizeofTScreen := sizeof (TScreen)
end;

function CSizeofScreenFormat: Cardinal;
asmname '__sizeof_screenformat';
function PSizeofTScreenFormat: Cardinal;
begin
  PSizeofTScreenFormat := sizeof (TScreenFormat)
end;

function CSizeofXWindowAttributes: Cardinal;
asmname '__sizeof_xwindowattributes';
function PSizeofTXWindowAttributes: Cardinal;
begin
  PSizeofTXWindowAttributes := sizeof (TXWindowAttributes)
end;

function CSizeofXSetWindowAttributes: Cardinal;
asmname '__sizeof_xsetwindowattributes';
function PSizeofTXSetWindowAttributes: Cardinal;
begin
  PSizeofTXSetWindowAttributes := sizeof (TXSetWindowAttributes)
end;

function CSizeofXHostAddress: Cardinal;
asmname '__sizeof_xhostaddress';
function PSizeofTXHostAddress: Cardinal;
begin
  PSizeofTXHostAddress := sizeof (TXHostAddress)
end;

function CSizeofXImage: Cardinal;
asmname '__sizeof_ximage';
function PSizeofTXImage: Cardinal;
begin
  PSizeofTXImage := sizeof (TXImage)
end;

function CSizeofXWindowChanges: Cardinal;
asmname '__sizeof_xwindowchanges';
function PSizeofTXWindowChanges: Cardinal;
begin
  PSizeofTXWindowChanges := sizeof (TXWindowChanges)
end;

function CSizeofXColor: Cardinal;
asmname '__sizeof_xcolor';
function PSizeofTXColor: Cardinal;
begin
  PSizeofTXColor := sizeof (TXColor)
end;

function CSizeofXKeyboardControl: Cardinal;
asmname '__sizeof_xkeyboardcontrol';
function PSizeofTXKeyboardControl: Cardinal;
begin
  PSizeofTXKeyboardControl := sizeof (TXKEyboardControl)
end;

function CSizeofXKeyboardState: Cardinal;
asmname '__sizeof_xkeyboardstate';
function PSizeofTXKeyboardState: Cardinal;
begin
  PSizeofTXKeyboardState := sizeof (TXKEyboardState)
end;

function CSizeofXModifierKeymap: Cardinal;
asmname '__sizeof_xmodifierkeymap';
function PSizeofTXModifierKeymap: Cardinal;
begin
  PSizeofTXModifierKeymap := sizeof (TXModifierKeymap)
end;

function CSizeofXKeyEvent: Cardinal;
asmname '__sizeof_xkeyevent';
function PSizeofTXKeyEvent: Cardinal;
begin
  PSizeofTXKeyEvent := sizeof (TXKeyEvent)
end;

function CSizeofXMotionEvent: Cardinal;
asmname '__sizeof_xmotionevent';
function PSizeofTXMotionEvent: Cardinal;
begin
  PSizeofTXMotionEvent := sizeof (TXMotionEvent)
end;

function CSizeofXCrossingEvent: Cardinal;
asmname '__sizeof_xcrossingevent';
function PSizeofTXCrossingEvent: Cardinal;
begin
  PSizeofTXCrossingEvent := sizeof (TXCrossingEvent)
end;

function CSizeofXFocusChangeEvent: Cardinal;
asmname '__sizeof_xfocuschangeevent';
function PSizeofTXFocusChangeEvent: Cardinal;
begin
  PSizeofTXFocusChangeEvent := sizeof (TXFocusChangeEvent)
end;

function CSizeofXGraphicsExposeEvent: Cardinal;
asmname '__sizeof_xgraphicsexposeevent';
function PSizeofTXGraphicsExposeEvent: Cardinal;
begin
  PSizeofTXGraphicsExposeEvent := sizeof (TXGraphicsExposeEvent)
end;

function CSizeofXClientMessageEvent: Cardinal;
asmname '__sizeof_xclientmessageevent';
function PSizeofTXClientMessageEvent: Cardinal;
begin
  PSizeofTXClientMessageEvent := sizeof (TXClientMessageEvent)
end;

function CSizeofXEvent: Cardinal;
asmname '__sizeof_xevent';
function PSizeofTXEvent: Cardinal;
begin
  PSizeofTXEvent := sizeof (TXEvent)
end;

function CSizeofXFontStruct: Cardinal;
asmname '__sizeof_xfontstruct';
function PSizeofTXFontStruct: Cardinal;
begin
  PSizeofTXFontStruct := sizeof (TXFontStruct)
end;

function CSizeofXEDataObject: Cardinal;
asmname '__sizeof_xedataobject';
function PSizeofTXEDataObject: Cardinal;
begin
  PSizeofTXEDataObject := sizeof (TXEDataObject)
end;

function CSizeofXmbTextItem: Cardinal;
asmname '__sizeof_xmbtextitem';
function PSizeofTXmbTextItem: Cardinal;
begin
  PSizeofTXmbTextItem := sizeof (TXmbTextItem)
end;

function CSizeofXwcTextItem: Cardinal;
asmname '__sizeof_xwctextitem';
function PSizeofTXwcTextItem: Cardinal;
begin
  PSizeofTXwcTextItem := sizeof (TXwcTextItem)
end;

function CSizeofXOMCharSetList: Cardinal;
asmname '__sizeof_xomcharsetlist';
function PSizeofTXOMCharSetList: Cardinal;
begin
  PSizeofTXOMCharSetList := sizeof (TXOMCharSetList)
end;

function CSizeofXOMOrientation: Cardinal;
asmname '__sizeof_xomorientation';
function PSizeofTXOMOrientation: Cardinal;
begin
  PSizeofTXOMOrientation := sizeof (TXOMOrientation)
end;

function CSizeofXOMFontInfo: Cardinal;
asmname '__sizeof_xomfontinfo';
function PSizeofTXOMFontInfo: Cardinal;
begin
  PSizeofTXOMFontInfo := sizeof (TXOMFontInfo)
end;

function CSizeofXVaNestedList: Cardinal;
asmname '__sizeof_xvanestedlist';
function  PSizeofXVaNestedList: Cardinal;
begin
  PSizeofXVaNestedList := sizeof (PXVaNestedList)
end;

function CSizeofXIMStyles: Cardinal;
asmname '__sizeof_ximstyles';
function  PSizeofXIMStyles: Cardinal;
begin
  PSizeofXIMStyles := sizeof (TXIMStyles)
end;

function CSizeofXIMCallback: Cardinal;
asmname '__sizeof_ximcallback';
function  PSizeofXIMCallback: Cardinal;
begin
  PSizeofXIMCallback := sizeof (TXIMCallback)
end;

function CSizeofXICCallback: Cardinal;
asmname '__sizeof_xiccallback';
function  PSizeofXICCallback: Cardinal;
begin
  PSizeofXICCallback := sizeof (TXICCallback)
end;

function CSizeofXIMText: Cardinal;
asmname '__sizeof_ximtext';
function  PSizeofXIMText: Cardinal;
begin
  PSizeofXIMText := sizeof (TXIMText)
end;

function CSizeofXIMPreeditStateNotifyCallbackStruct: Cardinal;
asmname '__sizeof_ximpreeditstatenotifycallbackstruct';
function  PSizeofXIMPreeditStateNotifyCallbackStruct: Cardinal;
begin
  PSizeofXIMPreeditStateNotifyCallbackStruct := sizeof
    (TXIMPreeditStateNotifyCallbackStruct)
end;

function CSizeofXIMStringConversionText: Cardinal;
asmname '__sizeof_ximstringconversiontext';
function  PSizeofXIMStringConversionText: Cardinal;
begin
  PSizeofXIMStringConversionText := sizeof (TXIMStringConversionText)
end;

function CSizeofXIMCaretDirection: Cardinal;
asmname '__sizeof_ximcaretdirection';
function  PSizeofXIMCaretDirection: Cardinal;
begin
  PSizeofXIMCaretDirection := sizeof (TXIMCaretDirection)
end;

function CSizeofXIMStringConversionCallbackStruct: Cardinal;
asmname '__sizeof_ximstringconversioncallbackstruct';
function  PSizeofXIMStringConversionCallbackStruct: Cardinal;
begin
  PSizeofXIMStringConversionCallbackStruct := sizeof
    (TXIMStringConversionCallbackStruct)
end;

function CSizeofXIMPreeditDrawCallbackStruct: Cardinal;
asmname '__sizeof_ximpreeditdrawcallbackstruct';
function  PSizeofXIMPreeditDrawCallbackStruct: Cardinal;
begin
  PSizeofXIMPreeditDrawCallbackStruct := sizeof
    (TXIMPreeditDrawCallbackStruct)
end;

function CSizeofXIMCaretStyle: Cardinal;
asmname '__sizeof_ximcaretstyle';
function  PSizeofXIMCaretStyle: Cardinal;
begin
  PSizeofXIMCaretStyle := sizeof (TXIMCaretStyle)
end;

function CSizeofXIMPreeditCaretCallbackStruct: Cardinal;
asmname '__sizeof_ximpreeditcaretcallbackstruct';
function  PSizeofXIMPreeditCaretCallbackStruct: Cardinal;
begin
  PSizeofXIMPreeditCaretCallbackStruct := sizeof
    (TXIMPreeditCaretCallbackStruct)
end;

function CSizeofXIMStatusDataType: Cardinal;
asmname '__sizeof_ximstatusdatatype';
function  PSizeofXIMStatusDataType: Cardinal;
begin
  PSizeofXIMStatusDataType := sizeof (TXIMStatusDataType)
end;

function CSizeofXIMStatusDrawCallbackStruct: Cardinal;
asmname '__sizeof_ximstatusdrawcallbackstruct';
function  PSizeofXIMStatusDrawCallbackStruct: Cardinal;
begin
  PSizeofXIMStatusDrawCallbackStruct := sizeof (TXIMStatusDrawCallbackStruct)
end;

function CSizeofXIMHotKeyTrigger: Cardinal;
asmname '__sizeof_ximhotkeytrigger';
function  PSizeofXIMHotKeyTrigger: Cardinal;
begin
  PSizeofXIMHotKeyTrigger := sizeof (TXIMHotKeyTrigger)
end;

function CSizeofXIMHotKeyTriggers: Cardinal;
asmname '__sizeof_ximhotkeytriggers';
function  PSizeofXIMHotKeyTriggers: Cardinal;
begin
  PSizeofXIMHotKeyTriggers := sizeof (TXIMHotKeyTriggers)
end;

function CSizeofXIMValuesList: Cardinal;
asmname '__sizeof_ximvaluelist';
function  PSizeofXIMValuesList: Cardinal;
begin
  PSizeofXIMValuesList := sizeof (TXIMValuesList)
end;

to begin do
  begin
    if CSizeofWCharT <> PSizeofTWCharT then
      begin
        WriteLn (StdErr, '(T)WCharT size mismatch!');
        Halt
      end;
    if CSizeofXExtData <> PSizeofTXExtData then
      begin
        WriteLn (StdErr, '(T)XExtData size mismatch!');
        Halt
      end;
    if CSizeofXExtCodes <> PSizeofTXExtCodes then
      begin
        WriteLn (StdErr, '(T)XExtCodes size mismatch!');
        Halt
      end;
    if CSizeofXPixmapFormatValues <> PSizeofTXPixmapFormatValues then
      begin
        WriteLn (StdErr, '(T)XPixmapFormatValues size mismatch!');
        Halt
      end;
    if CSizeofXGcValues <> PSizeofTXGcValues then
      begin
        WriteLn (StdErr, '(T)XGcValues size mismatch!');
        Halt
      end;
    if CSizeofVisual <> PSizeofTVisual then
      begin
        WriteLn (StdErr, '(T)Visual size mismatch!');
        Halt
      end;
    if CSizeofDepth <> PSizeofTDepth then
      begin
        WriteLn (StdErr, '(T)Depth size mismatch!');
        Halt
      end;
    if CSizeofScreen <> PSizeofTScreen then
      begin
        WriteLn (StdErr, '(T)Screen size mismatch!');
        Halt
      end;
    if CSizeofScreenFormat <> PSizeofTScreenFormat then
      begin
        WriteLn (StdErr, '(T)ScreenFormat size mismatch!');
        Halt
      end;
    if CSizeofXWindowAttributes <> PSizeofTXWindowAttributes then
      begin
        WriteLn (StdErr, '(T)XWindowAttributes size mismatch!');
        Halt
      end;
    if CSizeofXSetWindowAttributes <> PSizeofTXSetWindowAttributes then
      begin
        WriteLn (StdErr, '(T)XSetWindowAttributes size mismatch!');
        Halt
      end;
    if CSizeofXHostAddress <> PSizeofTXHostAddress then
      begin
        WriteLn (StdErr, '(T)XHostAddress size mismatch!');
        Halt
      end;
    if CSizeofXImage <> PSizeofTXImage then
      begin
        WriteLn (StdErr, '(T)XImage size mismatch!');
        Halt
      end;
    if CSizeofXWindowChanges <> PSizeofTXWindowChanges then
      begin
        WriteLn (StdErr, '(T)XWindowChanges size mismatch!');
        Halt
      end;
    if CSizeofXColor <> PSizeofTXColor then
      begin
        WriteLn (StdErr, '(T)XColor size mismatch!');
        Halt
      end;
    if CSizeofXKeyboardControl <> PSizeofTXKeyboardControl then
      begin
        WriteLn (StdErr, '(T)XKeyboardControl size mismatch!');
        Halt
      end;
    if CSizeofXKeyboardState <> PSizeofTXKeyboardState then
      begin
        WriteLn (StdErr, '(T)XKeyboardState size mismatch!');
        Halt
      end;
    if CSizeofXModifierKeymap <> PSizeofTXModifierKeymap then
      begin
        WriteLn (StdErr, '(T)XModifierKeymap size mismatch!');
        Halt
      end;
    if CSizeofXKeyEvent <> PSizeofTXKeyEvent then
      begin
        WriteLn (StdErr, '(T)XKeyEvent size mismatch!');
        Halt
      end;
    if CSizeofXMotionEvent <> PSizeofTXMotionEvent then
      begin
        WriteLn (StdErr, '(T)XMotionEvent size mismatch!');
        Halt
      end;
    if CSizeofXCrossingEvent <> PSizeofTXCrossingEvent then
      begin
        WriteLn (StdErr, '(T)XCrossingEvent size mismatch!');
        Halt
    end;
    if CSizeofXFocusChangeEvent <> PSizeofTXFocusChangeEvent then
      begin
        WriteLn (StdErr, '(T)XFocusChangeEvent size mismatch!');
        Halt
      end;
    if CSizeofXGraphicsExposeEvent <> PSizeofTXGraphicsExposeEvent then
      begin
        WriteLn (StdErr, '(T)XGraphicsExposeEvent size mismatch!');
        Halt
      end;
    if CSizeofXClientMessageEvent <> PSizeofTXClientMessageEvent then
      begin
        WriteLn (StdErr, '(T)XClientMessageEvent size mismatch!');
        Halt
      end;
    if CSizeofXEvent <> PSizeofTXEvent then
      begin
        WriteLn (StdErr, '(T)XEvent size mismatch!');
        Halt
      end;
    if CSizeofXFontStruct <> PSizeofTXFontStruct then
      begin
        WriteLn (StdErr, '(T)XFontStruct size mismatch!');
        Halt
      end;
    if CSizeofXEDataObject <> PSizeofTXEDataObject then
      begin
        WriteLn (StdErr, '(T)XEDataObject size mismatch!');
        Halt
      end;
    if CSizeofXmbTextItem <> PSizeofTXmbTextItem then
      begin
        WriteLn (StdErr, '(T)XmbTextItem size mismatch!');
        Halt
      end;
    if CSizeofXwcTextItem <> PSizeofTXwcTextItem then
      begin
        WriteLn (StdErr, '(T)XwcTextItem size mismatch!');
        Halt
      end;
    if CSizeofXOMCharSetList <> PSizeofTXOMCharSetList then
      begin
        WriteLn (StdErr, '(T)XOMCharSetList size mismatch!');
        Halt
      end;
    if CSizeofXOMOrientation <> PSizeofTXOMOrientation then
      begin
        WriteLn (StdErr, '(T)XOMOrientation size mismatch!');
        Halt
      end;
    if CSizeofXOMFontInfo <> PSizeofTXOMFontInfo then
      begin
        WriteLn (StdErr, '(T)XOMFontInfo size mismatch!');
        Halt
      end;
    if CSizeofXVaNestedList <> PSizeofXVaNestedList then
      begin
        WriteLn (StdErr, '(T)XVaNestedList size mismatch!');
        Halt
      end;
    if CSizeofXIMStyles <> PSizeofXIMStyles then
      begin
        WriteLn (StdErr, '(T)XIMStyles size mismatch!');
        Halt
      end;
    if CSizeofXIMCallback <> PSizeofXIMCallback then
      begin
        WriteLn (StdErr, '(T)XIMCallback size mismatch!');
        Halt
      end;
    if CSizeofXICCallback <> PSizeofXICCallback then
      begin
        WriteLn (StdErr, '(T)XICCallback size mismatch!');
        Halt
      end;
    if CSizeofXIMText <> PSizeofXIMText then
      begin
        WriteLn (StdErr, '(T)XIMText size mismatch!');
        Halt
      end;
    if CSizeofXIMPreeditStateNotifyCallbackStruct <> PSizeofXIMPreeditStateNotifyCallbackStruct then
      begin
        WriteLn (StdErr, '(T)XIMPreeditStateNotifyCallbackStruct size mismatch!');
        Halt
      end;
    if CSizeofXIMStringConversionText <> PSizeofXIMStringConversionText then
      begin
        WriteLn (StdErr, '(T)XIMStringConversionText size mismatch!');
        Halt
      end;
    if CSizeofXIMCaretDirection <> PSizeofXIMCaretDirection then
      begin
        WriteLn (StdErr, '(T)XIMCaretDirection size mismatch!');
        Halt
      end;
    if CSizeofXIMStringConversionCallbackStruct <> PSizeofXIMStringConversionCallbackStruct then
      begin
        WriteLn (StdErr, '(T)XIMStringConversionCallbackStruct size mismatch!');
        Halt
      end;
    if CSizeofXIMPreeditDrawCallbackStruct <> PSizeofXIMPreeditDrawCallbackStruct then
      begin
        WriteLn (StdErr, '(T)XIMPreeditDrawCallbackStruct size mismatch!');
        Halt
      end;
    if CSizeofXIMCaretStyle <> PSizeofXIMCaretStyle then
      begin
        WriteLn (StdErr, '(T)XIMCaretStyle size mismatch!');
        Halt
      end;
    if CSizeofXIMPreeditCaretCallbackStruct <> PSizeofXIMPreeditCaretCallbackStruct then
      begin
        WriteLn (StdErr, '(T)XIMPreeditCaretCallbackStruct size mismatch!');
        Halt
      end;
    if CSizeofXIMStatusDataType <> PSizeofXIMStatusDataType then
      begin
        WriteLn (StdErr, '(T)XIMStatusDataType size mismatch!');
        Halt
      end;
    if CSizeofXIMStatusDrawCallbackStruct <> PSizeofXIMStatusDrawCallbackStruct then
      begin
        WriteLn (StdErr, '(T)XIMStatusDrawCallbackStruct size mismatch!');
        Halt
      end;
    if CSizeofXIMHotKeyTrigger <> PSizeofXIMHotKeyTrigger then
      begin
        WriteLn (StdErr, '(T)XIMHotKeyTrigger size mismatch!');
        Halt
      end;
    if CSizeofXIMHotKeyTriggers <> PSizeofXIMHotKeyTriggers then
      begin
        WriteLn (StdErr, '(T)XIMHotKeyTriggers size mismatch!');
        Halt
      end;
    if CSizeofXIMValuesList <> PSizeofXIMValuesList then
      begin
        WriteLn (StdErr, '(T)XIMValuesList size mismatch!');
        Halt
      end;
  end;
end.
